// TREEMM: a software for identification of bipartite motifs in promoter sequences
// Copyright (C) 2011  Pierre NICOLAS
//
// This file is part of TREEMM.
//
//     TREEMM is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     Foobar is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with TREEMM.  If not, see <http://www.gnu.org/licenses/>.

#include <list>
#include <iostream>
#include <fstream>
#include <cstring>
#include <cstdlib>
#include <cmath>
#include <sstream>
#include <list>
//#include <pair>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_cdf.h>
using namespace std;

#include "Tree.h"
#include "Mmodel.h"

Mmodel::Mmodel(int nmotifs, bool allownomotif, double alpha_dirich_nuc, int rbackg, int lminspacer, int lmaxspacer, int lminvarspacer, int lmaxvarspacer, int lminbox1, int lmaxbox1, int lminbox2, int lmaxbox2, bool box2oblig, gsl_rng* r, Tree* tree)
{
  _r = r;
  _tree = tree;
  _nmotifs = nmotifs;
  _allownomotif = allownomotif;
  _lminspacer = lminspacer;
  _lmaxspacer = lmaxspacer;
  _lmaxbox1 = lmaxbox1;
  _lmaxbox2 = lmaxbox2;
  _lminbox1 = lminbox1;
  _lminbox2 = lminbox2;
  _box2oblig = box2oblig;
  _lminvarspacer = lminvarspacer;
  _lmaxvarspacer = lmaxvarspacer;
  _rbackg = rbackg;
  //_rbackg = 1;
  _maxdistfrom3end = 10;
  _alpha_evolontree = 1;
  _epsilon_evolontree = .5;
  _minpbox2 = 0.5;
  _alpha_dirich_nuc = alpha_dirich_nuc;
  // Jeffrey's prior for dirichlet 0.5 //1.;
  _dimlikcomput = _nmotifs;
  if (_allownomotif) {
    _dimlikcomput++;
  }

  _pmotifs = new double [_dimlikcomput];
  if (tree != NULL) {
    _pmotifs_ontree = new double [_dimlikcomput];    
  }
  _lspacer = new int [_nmotifs];
  _lvarspacer = new int [_nmotifs];
  _pvarspacer = new double* [_nmotifs];
  _lbox1 = new int [_nmotifs];
  _lbox2 = new int [_nmotifs];
  _pbox2 = new double [_nmotifs];
  _box2isoptional = new int [_nmotifs];
  _log_pmatbox1 = new double** [_nmotifs];
  _log_pmatbox2 = new double** [_nmotifs];
  _ref_posbox1 = new int [_nmotifs];
  _track_posbox1 = new int [_nmotifs];
  _track_posbox2 = new int [_nmotifs];
  double* alpha = new double [4];
  for (int ib=0; ib<4; ib++) {
    alpha[ib] = 1.; //_alpha_dirich_nuc;
  }

  _pmotifs[_dimlikcomput-1] = 1.0/(double)_dimlikcomput;
  if (tree != NULL) {
    _pmotifs_ontree[_dimlikcomput-1] = _pmotifs[_dimlikcomput-1];
  }
  for (int im=0; im<_nmotifs; im++) {
    _track_posbox1[im] = 0;
    _track_posbox2[im] = 0;
    _pmotifs[im] = 1.0/(double)_dimlikcomput;
    if (tree != NULL) {
      _pmotifs_ontree[im] = _pmotifs[im];
    }
    _lspacer[im] = (int)((double)_lmaxspacer/2.0);
    _lvarspacer[im] = _lmaxvarspacer;
    _pvarspacer[im] = new double [_lmaxvarspacer];
    for (int iv=0; iv<_lvarspacer[im]; iv++) {
      _pvarspacer[im][iv] = 1.0/(double)_lvarspacer[im];
    }
    _lbox1[im] = (int)((double)_lmaxbox1/2.0);
    _lbox2[im] = (int)((double)_lmaxbox2/2.0);
    if (_lbox1[im]<_lminbox1) {
      _lbox1[im] = _lminbox1;
    }
    if (_lbox2[im]<_lminbox2) {
      _lbox2[im] = _lminbox2;
    }
    _ref_posbox1[im] = (int)((double)_lbox1[im]/2.0);
    if (_ref_posbox1[im]<_lminbox1-1) {
      _ref_posbox1[im] = _lminbox1-1;
    }
    if (_box2oblig) {
      _pbox2[im] = 1;
      _box2isoptional[im] = 0;      
    } else {
      _pbox2[im] = 0.95;
      _box2isoptional[im] = 1;
    }
    _log_pmatbox1[im] = new double* [_lmaxbox1];
    _log_pmatbox2[im] = new double* [_lmaxbox2];
    for (int ipos=0; ipos<_lmaxbox1; ipos++) {
      _log_pmatbox1[im][ipos] = new double [4];
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox1[im][ipos]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox1[im][ipos][ib]=log(_log_pmatbox1[im][ipos][ib]);
      }
    }
    for (int ipos=0; ipos<_lmaxbox2; ipos++) {
      _log_pmatbox2[im][ipos] = new double [4];
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox2[im][ipos]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox2[im][ipos][ib]=log(_log_pmatbox2[im][ipos][ib]);
      }
    }
  }
  _log_backgroundM1 = new double [20];
  _count_nuc_matbox1 = new int** [_nmotifs];
  _count_nuc_matbox2 = new int** [_nmotifs];
  _count_varspacer = new int* [_nmotifs];
  _count_box2nobox2 = new int* [_nmotifs];
  for (int im=0; im<_nmotifs; im++) {
    _count_varspacer[im] = new int [_lmaxvarspacer];
    _count_box2nobox2[im] = new int [2];
    _count_nuc_matbox1[im] = new int* [_lmaxbox1];
    _count_nuc_matbox2[im] = new int* [_lmaxbox2];
    for (int ipos=0; ipos<_lmaxbox1; ipos++) {
      _count_nuc_matbox1[im][ipos] = new int [4];
    }
    for (int ipos=0; ipos<_lmaxbox2; ipos++) {
      _count_nuc_matbox2[im][ipos] = new int [4];
    }
  }
  delete [] alpha;

  int codemax = (int)((pow(4,_rbackg+2)-4.0)/3.0);
  cout << "codemax " << codemax << endl;
  _log_backgroundMr = new double [codemax];
  _count_nuc_backg = new int [codemax];

  _invtemp = 1.0;
}

Mmodel::~Mmodel()
{
}

void Mmodel::computeBackgroundM1(Sequences* seqs)
{
  int* countsM1 = new int [20];
  for (int i=0; i<20; i++) {
    countsM1[i]=0;
  }
  for (int iseq=0; iseq<seqs->NbSeq(); iseq++) {
    int* tseq = *(seqs->sequencesMr(1)+iseq);
    for (int t=0; t <seqs->LeSeq(); t++) {
      countsM1[tseq[t]]++;
    }
  }
  for (int i=0; i<4; i++) {
      _log_backgroundM1[i] = log(1.0/4.0);    
  }
  int ci=4;
  while (ci<20) {
    double tsum=0;
    //cout << "counts";
    for (int i=ci+0; i<ci+4; i++) {
      tsum+=countsM1[i];
      //cout << " " << countsM1[i]; 
    }  
    //cout << endl;
    //cout << "pobs:";
    for (int i=ci+0; i<ci+4; i++) {

      _log_backgroundM1[i] = log((double)countsM1[i]/(double)tsum);
      //_log_backgroundM1[i] = log(1.0/4.0);
      //_log_backgroundM1[i]=0; //NODATA simulation
      //cout << " " << _log_backgroundM1[i]; 
    }      
    //cout << endl;
    ci+=4;
  }
  delete [] countsM1;
}

void Mmodel::computeBackgroundMr(Sequences* seqs)
{
  int codemax = (int)((pow(4,_rbackg+2)-4.0)/3.0);
  for (int ic=0; ic<codemax; ic++) {
    _count_nuc_backg[ic]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    int* tseqMr = *(seqs->sequencesMr(_rbackg)+iseq);
    for (int t=0; t<_lseq; t++) {
      _count_nuc_backg[tseqMr[t]]++;
    }
  }
  int codemaxinfr = (int)((pow(4,_rbackg+1)-4.0)/3.0);
  for (int ic=0; ic<codemaxinfr; ic++) {
    _log_backgroundMr[ic] = log(1.0/4.0);
  }
  int ici=codemaxinfr;
  while (ici<codemax) {
    double tsum=0;
    //cout << "counts ici=" << ici;
    for (int i=ici+0; i<ici+4; i++) {
      tsum+=_count_nuc_backg[i];
      //cout << " " << _count_nuc_backg[i]; 
    }  
    //cout << endl;
    //cout << "pobs:";
    for (int i=ici+0; i<ici+4; i++) {

      _log_backgroundMr[i] = log((double)_count_nuc_backg[i]/(double)tsum);
      //_log_backgroundMr[i] = log(1.0/4.0);
      //_log_backgroundMr[i]=0; //NODATA simulation
      //cout << " " << _log_backgroundMr[i]; 
    }      
    //cout << endl;
    ici+=4;
  }
}

void Mmodel::MemAllocAlgo(Sequences* seqs)
{
  _nseqs = seqs->NbSeq();
  _lseq = seqs->LeSeq();

  this->computeBackgroundM1(seqs);
  this->computeBackgroundMr(seqs);

  _alloc_imotif = new int [_nseqs];
  _alloc_tbox1refpos = new int [_nseqs];
  _alloc_varspacer = new int [_nseqs];

  _logl_cumsum_background_seq = new double* [_nseqs];
  _logl_seq_by_motif = new double* [_nseqs];
  _logl_ratio_box1 = new double** [_nseqs];
  _logl_ratio_box2 = new double** [_nseqs];
  _ratio_seqt = new double** [_nseqs];
  _tmp_ratio_seqt  = new double* [_nseqs];
  for (int iseq=0; iseq<_nseqs; iseq++) {
    _logl_seq_by_motif[iseq] = new double [_dimlikcomput];
    _logl_ratio_box1[iseq] = new double* [_nmotifs];
    _logl_ratio_box2[iseq] = new double* [_nmotifs];
    _ratio_seqt[iseq] = new double* [_nmotifs];
    _logl_cumsum_background_seq[iseq] = new double [_lseq];
    for (int im=0; im<_nmotifs; im++) {
      _logl_ratio_box1[iseq][im] = new double [_lseq];
      _logl_ratio_box2[iseq][im] = new double [_lseq];
      _ratio_seqt[iseq][im] = new double [_lseq];
    }
    _tmp_ratio_seqt[iseq] = new double [_lseq];
  }  
  _p_pos_motif = new double [_lseq];
  _breakpoints_p_pos_motif = new int [_lseq];
  _count_pos_motif = new int [_lseq];
  _nbins_p_pos_motif = 1;
  for (int tbox1refpos=0; tbox1refpos<_lseq; tbox1refpos++) {
    _breakpoints_p_pos_motif[tbox1refpos]=0;
    if (tbox1refpos>=_lmaxbox1 && tbox1refpos<_lseq-_maxdistfrom3end) {
      _p_pos_motif[tbox1refpos]=1.0/(double)(_lseq-_maxdistfrom3end-_lmaxbox1);
    } else {
      _p_pos_motif[tbox1refpos]=0;
    }
  }
}

void Mmodel::Compute_logl_cumsum_background_seq(Sequences* seqs)
{
  for (int iseq=0; iseq<_nseqs; iseq++) {
    int* tseq = *(seqs->sequencesMr(_rbackg)+iseq);
    _logl_cumsum_background_seq[iseq][0] = _log_backgroundMr[tseq[0]];
    for (int t=1; t<_lseq; t++) {
      _logl_cumsum_background_seq[iseq][t] = 
	_logl_cumsum_background_seq[iseq][t-1] + _log_backgroundMr[tseq[t]]; 
    }
    if (_allownomotif) {
      _logl_seq_by_motif[iseq][_dimlikcomput-1] = _logl_cumsum_background_seq[iseq][_lseq-1];
    }
  }
}

void Mmodel::InitAlgo() 
{
  for (int iseq=0; iseq<_nseqs; iseq++) {
    _alloc_imotif[iseq] = this->sampleProbVect(_dimlikcomput, _pmotifs);
    if (_alloc_imotif[iseq]<_nmotifs) {
      _alloc_tbox1refpos[iseq] = _lmaxbox1+gsl_rng_uniform_int(_r,_lseq-_maxdistfrom3end-_lmaxbox1);
      if (gsl_ran_bernoulli(_r, _pbox2[_alloc_imotif[iseq]])==0) {
	_alloc_varspacer[iseq] = -1;
      } else {
	_alloc_varspacer[iseq] = this->sampleProbVect(_lvarspacer[_alloc_imotif[iseq]], _pvarspacer[_alloc_imotif[iseq]]);
      }
    } else {
      _alloc_tbox1refpos[iseq] = -1;
      _alloc_varspacer[iseq] = -1;
    }
    
    //cout << "_alloc_imotif[iseq]" << _alloc_imotif[iseq] << " _alloc_varspacer[iseq] " << _alloc_varspacer[iseq] << " _alloc_tbox1refpos[iseq] " << _alloc_tbox1refpos[iseq] << endl;
  }  
}

void Mmodel::Update_pmatbox(Sequences* seqs) 
{
  for (int im=0; im<_nmotifs; im++) {
    for (int ipos=0; ipos<_lbox1[im]; ipos++) {
      for (int ib=0; ib<4; ib++) {
	_count_nuc_matbox1[im][ipos][ib] = 0;
      }
    }
    for (int ipos=0; ipos<_lbox2[im]; ipos++) {
      for (int ib=0; ib<4; ib++) {
	_count_nuc_matbox2[im][ipos][ib] = 0;
      }
    }
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    if (_alloc_imotif[iseq] < _nmotifs) {
      int* tseq = *(seqs->sequencesMr(0)+iseq);
      int im = _alloc_imotif[iseq];
      
      int b1tmin = tbox1_from_tbox1refpos(_ref_posbox1[im],_alloc_tbox1refpos[iseq]);
      int b1tmax = b1tmin + _lbox1[im] - 1;
      int b1ipos = 0;
      for (int t = b1tmin; t <= b1tmax; t++) {
	if (t>=0 && t<_lseq) {
	  _count_nuc_matbox1[im][b1ipos][tseq[t]]++; //NODATA
	}
	b1ipos++;
      }
      
      if (_alloc_varspacer[iseq]>=0) { // otherwise no box2
	int b2tmax = b1tmin - 1 - _lspacer[im] - _alloc_varspacer[iseq];
	int b2tmin = b2tmax - _lbox2[im] + 1;
	int b2ipos = 0;
	for (int t = b2tmin; t<=b2tmax; t++) {
	  if (t>=0 && t<_lseq) {
	    _count_nuc_matbox2[im][b2ipos][tseq[t]]++; //NODATA
	  }
	  b2ipos++;
	}
      }
    }
  }
    
    
  double* alpha = new double [4];
  for (int im=0; im<_nmotifs; im++) {
    for (int ipos=0; ipos<_lbox1[im]; ipos++) {
      for (int ib=0; ib<4; ib++) {
	alpha[ib] = (double)_count_nuc_matbox1[im][ipos][ib]+_alpha_dirich_nuc;
      }
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox1[im][ipos]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox1[im][ipos][ib]=log(_log_pmatbox1[im][ipos][ib]);
      }
    }
    for (int ipos=0; ipos<_lbox2[im]; ipos++) {
      for (int ib=0; ib<4; ib++) {
	alpha[ib] = (double)_count_nuc_matbox2[im][ipos][ib]+_alpha_dirich_nuc;
      }
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox2[im][ipos]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox2[im][ipos][ib]=log(_log_pmatbox2[im][ipos][ib]);
      }
    }
  }    
  delete [] alpha;

//   for (int im=0; im<_nmotifs; im++) {
//     cout << "motif " << im << endl;
//     cout << "  box1" << endl;
//     for (int ipos=0; ipos<_lbox1[im]; ipos++) {
//       cout << "  ";
//       for (int ib=0; ib<4; ib++) {
// 	cout << " " << _count_nuc_matbox1[im][ipos][ib];
//       }
//       for (int ib=0; ib<4; ib++) {
// 	cout << " " << exp(_log_pmatbox1[im][ipos][ib]);
//       }
//     }
//     cout << "  box2" << endl;
//     for (int ipos=0; ipos<_lbox2[im]; ipos++) {
//       cout << "  ";
//       for (int ib=0; ib<4; ib++) {
// 	cout << " " << _count_nuc_matbox2[im][ipos][ib];
//       }
//       for (int ib=0; ib<4; ib++) {
// 	cout << " " << exp(_log_pmatbox2[im][ipos][ib]);
//       }
//       cout << endl;
//     }
//   }    
}

double Mmodel::Compute_loglik_p_pos_motif(int *breakpoints) 
{
  double combin = 1;
  int npos = _lseq - _maxdistfrom3end - _lmaxbox1;
  int nbreaks=0;
  for (int t=_lmaxbox1; t<_lseq-_maxdistfrom3end; t++) {
    if (breakpoints[t]==1) {
      nbreaks++;
      combin *= (double)((npos-1) - (nbreaks-1))/(double)(nbreaks);
    }
  }  
  int nbins = nbreaks+1;
  double* alpha = new double [nbins];
  int* widthbin = new int [nbins];
  for (int ibin=0; ibin <nbins; ibin++) {
    alpha[ibin]=1.0;
    widthbin[ibin]=0;
  }
  int ibin=0;
  for (int t=_lmaxbox1; t<_lseq-_maxdistfrom3end; t++) {
    if (breakpoints[t]==1) {
      ibin++;
    }
    alpha[ibin]+=(double)_count_pos_motif[t];
    widthbin[ibin]++;	
  }  
  double asum = 0;
  for (int ibin=0; ibin <nbins; ibin++) {
    asum += alpha[ibin];
  }
  
  double loglik = log(1.0/combin) + gsl_sf_lngamma((double)nbins) - gsl_sf_lngamma(asum);
  for (int ibin=0; ibin <nbins; ibin++) {
    loglik += -(alpha[ibin]-1.0)*log((double)widthbin[ibin]) + gsl_sf_lngamma(alpha[ibin]) - gsl_sf_lngamma(1.0);
  }
  delete [] alpha;
  //cout << "loglik " << loglik << endl;
  return loglik;
}

void Mmodel::Update_p_pos_motif() 
{
  // Counts
  for (int t=0; t<_lseq; t++) {
    _count_pos_motif[t]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    if (_alloc_imotif[iseq]<_nmotifs) {
      _count_pos_motif[_alloc_tbox1refpos[iseq]]++;
    }    
  }

  // Model change
  int npos = _lseq- _maxdistfrom3end - _lmaxbox1;
  int newnbins = _nbins_p_pos_motif;
  int* newbreakpoints = new int [_lseq];
  for (int t=0; t<_lseq; t++) {
    newbreakpoints[t] = _breakpoints_p_pos_motif[t];
  }
  double logpaccept = 0;
  int typemove = 2*gsl_ran_bernoulli(_r, 0.5)-1;
  if (typemove==-1 && _nbins_p_pos_motif>1) { 
    // attempt to remove a breakpoint
    newnbins--;
    int breaktoberemoved = gsl_rng_uniform_int(_r,_nbins_p_pos_motif-1);
    int ibreak=-1;
    int t=_lmaxbox1-1;// no breakpoint allowed in pos _lmaxbox1-1;
    while (ibreak!=breaktoberemoved) {
      t++;
      if (_breakpoints_p_pos_motif[t]==1) {
	ibreak++;
      }
    }
    newbreakpoints[t]=0;
    double diffloglik = this->Compute_loglik_p_pos_motif(newbreakpoints) 
      - this->Compute_loglik_p_pos_motif(_breakpoints_p_pos_motif);
    double difflogproposal = log(1.0/(double)((npos-1) - (newnbins-1))) 
      - log(1.0/(double)(_nbins_p_pos_motif-1));
    logpaccept = diffloglik + difflogproposal;
  } else if (typemove==1 && _nbins_p_pos_motif<npos-1) {
    // attempt to add a breakpoint
    newnbins++;
    int rsel = gsl_rng_uniform_int(_r,(npos-1) - (_nbins_p_pos_motif-1));
    int t=_lmaxbox1; // no breakpoint allowed in pos _lmaxbox1;
    int icount = -1;
    while (icount != rsel) {
      t++;
      if (_breakpoints_p_pos_motif[t]==0) {
	icount++;
      }
    }
    newbreakpoints[t]=1;
    double diffloglik = this->Compute_loglik_p_pos_motif(newbreakpoints) 
      - this->Compute_loglik_p_pos_motif(_breakpoints_p_pos_motif);
    double difflogproposal = log(1.0/(double)(newnbins - 1)) 
      - log(1.0/(double)((npos-1) - (_nbins_p_pos_motif-1)));
    logpaccept = diffloglik + difflogproposal;
  }

  //cout << "before _nbins_p_pos_motif " << _nbins_p_pos_motif << endl;
  if (gsl_ran_bernoulli(_r, exp(logpaccept))==1) {
    delete [] _breakpoints_p_pos_motif;
    _breakpoints_p_pos_motif = newbreakpoints;
    _nbins_p_pos_motif = newnbins;
  } else {
    delete [] newbreakpoints;
  }
  //cout << "after _nbins_p_pos_motif " << _nbins_p_pos_motif << endl;

  // Parameter update
  if (_nbins_p_pos_motif>1) {
    double* alpha = new double [_nbins_p_pos_motif];
    double* newpbin = new double [_nbins_p_pos_motif];
    int* widthbin = new int [_nbins_p_pos_motif];
    for (int ibin=0; ibin <_nbins_p_pos_motif; ibin++) {
      alpha[ibin]=1.0;
      widthbin[ibin]=0;
    }
    int ibin=0;
    for (int t=_lmaxbox1; t<_lseq-_maxdistfrom3end; t++) {
      if (_breakpoints_p_pos_motif[t]==1) {
	ibin++;
      }
      alpha[ibin]+=_count_pos_motif[t];
      widthbin[ibin]++;	
    }      
    gsl_ran_dirichlet(_r, _nbins_p_pos_motif, alpha, newpbin);
    for (int ibin=0; ibin<_nbins_p_pos_motif; ibin++) {
      //cout << "widthbin["<<ibin<<"] " << widthbin[ibin]  << " alpha["<<ibin<<"] " << alpha[ibin] << endl;
    }
    ibin=0;
    for (int t=_lmaxbox1; t<_lseq-_maxdistfrom3end; t++) {
      if (_breakpoints_p_pos_motif[t]==1) {
	ibin++;
      }
      _p_pos_motif[t] = newpbin[ibin]/(double)widthbin[ibin];
      //cout << _p_pos_motif[t] << " " << newpbin[ibin] << "/" << (double)widthbin[ibin] << endl;
    }   
    delete [] alpha;
    delete [] newpbin;
    delete [] widthbin;         
  } else if (_nbins_p_pos_motif == 1) {
    for (int t=_lmaxbox1; t<_lseq-_maxdistfrom3end; t++) {
      _p_pos_motif[t] = 1.0/(double)(_lseq-_maxdistfrom3end-_lmaxbox1);
    }    
  }
  //cout << endl;


  if (0) {
    double* alpha = new double [_lseq];
    for (int t=0; t<_lseq; t++) {
      alpha[t] = 1.0 + _count_pos_motif[t];
    }
    gsl_ran_dirichlet(_r, _lseq-_lmaxbox1-_maxdistfrom3end, 
		      alpha + _lmaxbox1, _p_pos_motif + _lmaxbox1);
    delete [] alpha;
  }
}

void Mmodel::Update_pvarspacer() 
{
  for (int im=0; im<_nmotifs; im++) {
    _count_box2nobox2[im][0]=0;
    _count_box2nobox2[im][1]=0;
    for (int ivs=0; ivs<_lmaxvarspacer; ivs++) {
      _count_varspacer[im][ivs]=0;
    }
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    if (_alloc_imotif[iseq]<_nmotifs) {
      if (_alloc_varspacer[iseq]>=0) {
	_count_box2nobox2[_alloc_imotif[iseq]][0]++;
	_count_varspacer[_alloc_imotif[iseq]][_alloc_varspacer[iseq]]++;
      } else {
	_count_box2nobox2[_alloc_imotif[iseq]][1]++;
      }
    }
  }

  double* alpha = new double [_lmaxvarspacer];
  for (int im=0; im<_nmotifs; im++) {
    for (int ivs=0; ivs<_lvarspacer[im]; ivs++) {
      alpha[ivs]=1+_count_varspacer[im][ivs];
    }        
    gsl_ran_dirichlet(_r, _lvarspacer[im], alpha, _pvarspacer[im]);
//     if (_box2isoptional[im]==1) {
//       double newpbox2 = _pbox2[im] + gsl_ran_ugaussian(_r)*0.1;
//       if (newpbox2 >= _minpbox2 && newpbox2 <= 1) {
// 	double logpaccept = (log(newpbox2)-log(_pbox2[im]))*(double)_count_box2nobox2[im][0] + (log(1.0-newpbox2)-log(1.0-_pbox2[im]))*(double)_count_box2nobox2[im][1];
// 	if (gsl_ran_bernoulli(_r, exp(logpaccept))==1) {
// 	  _pbox2[im]= newpbox2;
// 	}
//       }
//     }
    if (_box2isoptional[im]==1) {
      double newpbox2 = gsl_ran_beta(_r, 1.0+_count_box2nobox2[im][0], 
				     1.0+_count_box2nobox2[im][1]);
      if (newpbox2 >= _minpbox2) {
	_pbox2[im]= newpbox2;
      }
    }
//     cout << "motif " << im << " _pbox2[im] " << _pbox2[im] << " _pvarspacer[im]";
//     for (int ivs=0; ivs<_lvarspacer; ivs++) {
//       cout << " " << _pvarspacer[im][ivs];
//     }        
//     cout << endl;
  }
  delete [] alpha;
}

void Mmodel::Update_pmotifs(bool update_probavect)
{
  if (_tree==NULL) {
    double* alpha = new double [_dimlikcomput];
    for (int im=0; im<_dimlikcomput; im++) {
      alpha[im]=1;
    }  
    for (int iseq=0; iseq<_nseqs; iseq++) {
      alpha[_alloc_imotif[iseq]]++;
    }    
    if (update_probavect) {
      gsl_ran_dirichlet(_r, _dimlikcomput, alpha, _pmotifs); 
//       if (!_allownomotif) {
// 	gsl_ran_dirichlet(_r, _dimlikcomput, alpha, _pmotifs);  
//       } else {
// 	gsl_ran_dirichlet(_r, _dimlikcomput-1, alpha, _pmotifs);  
// 	double a=alpha[_dimlikcomput-1];
// 	double b=1+(double)_nseqs - (alpha[_dimlikcomput-1]-1);
// 	_pmotifs[_dimlikcomput-1]=gsl_ran_beta(_r, a, b);
// 	for (int im=0; im<_dimlikcomput-1; im++) {
// 	  _pmotifs[im]=_pmotifs[im]*(1-_pmotifs[_dimlikcomput-1]);
// 	}  
//       }
    }
    delete [] alpha;
  } else {
    if (update_probavect) {
      //_tree->Update_alpha_and_pmotifs(&_epsilon_evolontree, &_alpha_evolontree, _pmotifs, _pmotifs_ontree);
      _tree->Update_alpha_and_pmotifs_undistinct(&_epsilon_evolontree, &_alpha_evolontree, _pmotifs, _pmotifs_ontree);
//       if (!_allownomotif) {
// 	_tree->Update_alpha_and_pmotifs_undistinct(&_epsilon_evolontree, &_alpha_evolontree, _pmotifs, _pmotifs_ontree);
//       } else {
// 	_tree->Update_alpha_and_pmotifs_undistinct_pnomotif(&_epsilon_evolontree, &_alpha_evolontree, _pmotifs, _pmotifs_ontree);
//       }
    } else {
      //_tree->Update_alpha_and_pmotifs(&_epsilon_evolontree, &_alpha_evolontree, NULL, NULL);
      _tree->Update_alpha_and_pmotifs(&_epsilon_evolontree, &_alpha_evolontree, _pmotifs, NULL);
    }
  }
}

void Mmodel::Compute_logl_ratio_boxes(Sequences* seqs)
{
  for (int im=0; im<_nmotifs; im++) {
    this->Compute_logl_ratio_box1(seqs, im);
    this->Compute_logl_ratio_box2(seqs, im);
  }
  

//   cout << "_logl_cumsum_background_seq[0]";
//   for (int t=0; t<_lseq; t++) {
//     cout << " " << t << ":" << _logl_cumsum_background_seq[0][t];
//   }  
//   cout << endl;
//   cout << "_logl_ratio_box1[0][0]";
//   for (int t=0; t<_lseq; t++) {
//     cout << " " << t << ":" << _logl_ratio_box1[0][0][t];
//   }  
//   cout << endl;
}

void Mmodel::Compute_logl_ratio_box1(Sequences* seqs, int imotif)
{
  for (int iseq=0; iseq<_nseqs; iseq++) {
    int* tseq = *(seqs->sequencesMr(0)+iseq);
    for (int t=0; t<_lseq; t++) {
      int xmin=t;
      int xmax=t+_lbox1[imotif]-1;
      if (xmax>_lseq-1) {
	xmax=_lseq-1;
      }
      _logl_ratio_box1[iseq][imotif][t] = - _logl_cumsum_background_seq[iseq][xmax];
      if (xmin>0) {
	_logl_ratio_box1[iseq][imotif][t] += _logl_cumsum_background_seq[iseq][xmin-1];
      }
      int ipos=0;
      for (int x=xmin; x<=xmax; x++) {
	_logl_ratio_box1[iseq][imotif][t] += _log_pmatbox1[imotif][ipos][tseq[x]];	
	//_logl_ratio_box1[iseq][imotif][t] += 0; // NODATA
	ipos++;
      }
    }      
  }  
}


void Mmodel::Compute_logl_ratio_box2(Sequences* seqs, int imotif)
{
  for (int iseq=0; iseq<_nseqs; iseq++) {
    int* tseq = *(seqs->sequencesMr(0)+iseq);
    for (int t=0; t<_lseq; t++) {
      int xmax=t;
      int xmin=t-_lbox2[imotif]+1;
      if (xmin<0) {
	xmin=0;
      }
      _logl_ratio_box2[iseq][imotif][t] = - _logl_cumsum_background_seq[iseq][xmax];
      if (xmin>0) {
	_logl_ratio_box2[iseq][imotif][t] += _logl_cumsum_background_seq[iseq][xmin-1];
      }
      int ipos=_lbox2[imotif]-1;
      for (int x=xmax; x>=xmin; x--) {
	_logl_ratio_box2[iseq][imotif][t] += _log_pmatbox2[imotif][ipos][tseq[x]];	
	//_logl_ratio_box2[iseq][imotif][t] += 0; // NODATA
	ipos--;
      }
    }      
  }  
}

void Mmodel::Compute_ratio_seqt()
{
  for (int imotif=0; imotif<_nmotifs; imotif++) {
    this->Compute_ratio_seqt_for_motif(imotif);
  }
}

void Mmodel::Compute_ratio_seqt_for_motif(int imotif)
{
  double* xvarsp = new double [_lvarspacer[imotif]];
  for (int iseq=0; iseq<_nseqs; iseq++) {
    for (int tbox1=0; tbox1<_lseq; tbox1++) {
      double xbox1 = _logl_ratio_box1[iseq][imotif][tbox1];
      double xnobox2 = xbox1 + log(1-_pbox2[imotif]);
      for (int ivs=0; ivs<_lvarspacer[imotif]; ivs++) {
	xvarsp[ivs] = xbox1 
	  + log(_pbox2[imotif]*_pvarspacer[imotif][ivs]);
	int tbox2 = tbox1-_lspacer[imotif]-1-ivs;
	if (tbox2>=0) {
	  xvarsp[ivs] += _logl_ratio_box2[iseq][imotif][tbox2];
	}
      }
      double tpsum=exp(xnobox2);
      for (int ivs=0; ivs<_lvarspacer[imotif]; ivs++) {
	tpsum += exp(xvarsp[ivs]);
      }
      _ratio_seqt[iseq][imotif][tbox1] = tpsum;
    }  
  }
  delete [] xvarsp;
}

void Mmodel::Update_Switchvarspacer()
{
  int imotif = gsl_rng_uniform_int(_r,_nmotifs);
  int iside = -1 + 2*gsl_ran_bernoulli(_r, 0.5);
  double pthresh = 0.1;

  //  cout << "Mmodel::Update_Switchvarspacer() imotif "<< imotif << " iside " << iside << endl;
  if ((_lspacer[imotif]-iside <=_lmaxspacer && _lspacer[imotif]-iside >= _lminspacer) 
      && ((iside<0 && _pvarspacer[imotif][0]<pthresh) 
	  || (iside>0 && _pvarspacer[imotif][_lvarspacer[imotif]-1]<pthresh)) ) {
    double* old_pvarspacer = new double [_lmaxvarspacer];
    for (int ivs=0; ivs<_lmaxvarspacer; ivs++) {
      old_pvarspacer[ivs]=_pvarspacer[imotif][ivs];
    }
    int old_lspacer = _lspacer[imotif];
    _lspacer[imotif] -= iside;
    if (iside<0) {
      for (int ivs=1; ivs<_lvarspacer[imotif]; ivs++) {
	_pvarspacer[imotif][ivs-1]=old_pvarspacer[ivs];
      }      
      _pvarspacer[imotif][_lvarspacer[imotif]-1]=old_pvarspacer[0];
    }
    if (iside>0) {
      for (int ivs=0; ivs<_lvarspacer[imotif]-1; ivs++) {
	_pvarspacer[imotif][ivs+1]=old_pvarspacer[ivs];
      }      
      _pvarspacer[imotif][0]=old_pvarspacer[_lvarspacer[imotif]-1];
    }
//     cout << "_lspacer " << old_lspacer << " -> " << _lspacer[imotif] << endl;
//     for (int ivs=0; ivs<_lvarspacer[imotif]; ivs++) {
//       cout << ivs << " " << old_pvarspacer[ivs] << " -> " << _pvarspacer[imotif][ivs]<< endl;
//     }
    double oldlogltot = _logltot;
    double* old_loglseq = new double [_nseqs];
    int* old_alloc_imotif = new int [_nseqs];
    for (int iseq=0; iseq<_nseqs; iseq++) {
      old_loglseq[iseq] = _logl_seq_by_motif[iseq][imotif];
      old_alloc_imotif[iseq] = _alloc_imotif[iseq];
      for (int t=0; t<_lseq; t++) {
	_tmp_ratio_seqt[iseq][t] = _ratio_seqt[iseq][imotif][t];
      }
    }
    this->Compute_ratio_seqt_for_motif(imotif);
    this->Compute_logl_seq_for_motif(imotif);
    this->Compute_logltot_and_allocimotif();

    double paccept = exp(_logltot - oldlogltot);
    //    cout << "paccept " << paccept << endl;
    if (gsl_ran_bernoulli(_r, paccept)==1) {
      //cout << "accepted" << endl;
    } else {
      //cout << "rejected" << endl; 
      _logltot = oldlogltot;
      for (int ivs=0; ivs<_lmaxvarspacer; ivs++) {
	_pvarspacer[imotif][ivs] = old_pvarspacer[ivs];
      }
      _lspacer[imotif] = old_lspacer;
      for (int iseq=0; iseq<_nseqs; iseq++) {
	_logl_seq_by_motif[iseq][imotif] = old_loglseq[iseq];
	_alloc_imotif[iseq] = old_alloc_imotif[iseq];
	for (int t=0; t<_lseq; t++) {
	  _ratio_seqt[iseq][imotif][t]=_tmp_ratio_seqt[iseq][t];
	}
      }
    }
    delete [] old_loglseq;
    delete [] old_pvarspacer;
    delete [] old_alloc_imotif;
  }
}

void Mmodel::Update_Sizevarspacer()
{
  int imotif = gsl_rng_uniform_int(_r,_nmotifs);
  int iside = -1 + 2*gsl_ran_bernoulli(_r, 0.5); // -1 left, +1 right
  int idirection = -1 + 2*gsl_ran_bernoulli(_r, 0.5); // +1 add, -1 remove
  double pthresh = 0.3/(double)_lmaxvarspacer;

  bool nonvalide = false;
  nonvalide = nonvalide 
    || ((idirection==-1) && (iside==-1) && (_pvarspacer[imotif][0]>pthresh));
  nonvalide = nonvalide 
    || ((idirection==-1) && (iside==1) && (_pvarspacer[imotif][_lvarspacer[imotif]-1]>pthresh));
  nonvalide = nonvalide
    || ((idirection==-1) && (iside==-1) && (_lspacer[imotif]+1>_lmaxspacer));
  nonvalide = nonvalide 
    || ((idirection==1) && (_lvarspacer[imotif]+1>_lmaxvarspacer));
  nonvalide = nonvalide 
    || ((idirection==1) && (iside==-1) && (_lspacer[imotif]-1<_lminspacer));
  nonvalide = nonvalide 
    || ((idirection==-1) && (_lvarspacer[imotif]-1<_lminvarspacer));
  
  if (!nonvalide) {
    //cout << "Mmodel::Update_Sizevarspacer() imotif "<< imotif << " iside " << iside << " idirection " << idirection << endl;
    int old_lspacer = _lspacer[imotif];
    int old_lvarspacer = _lvarspacer[imotif];
    double* old_pvarspacer = new double [_lmaxvarspacer];
    double* alpha = new double [_lmaxvarspacer];
    for (int ivs=0; ivs<_lmaxvarspacer; ivs++) {
      old_pvarspacer[ivs]=_pvarspacer[imotif][ivs];
      alpha[ivs] = 1.0;
    }
    
    _lvarspacer[imotif] += idirection;
    if (iside==-1) {
      _lspacer[imotif] -= idirection;
    }

    double oldval = -1;
    double newval = -1;
    if (iside==1 && idirection==1) {
      newval = gsl_rng_uniform(_r)*pthresh;
      _pvarspacer[imotif][_lvarspacer[imotif]-1]=newval;
      for (int ivs=0; ivs<_lvarspacer[imotif]-1; ivs++) {
	_pvarspacer[imotif][ivs]=old_pvarspacer[ivs]*(1.0-newval);
      }      
    }
    if (iside==1 && idirection==-1) {
      oldval = old_pvarspacer[_lvarspacer[imotif]];
      for (int ivs=0; ivs<_lvarspacer[imotif]; ivs++) {
	_pvarspacer[imotif][ivs]=old_pvarspacer[ivs]/(1.0-oldval);
      }      
    }
    if (iside==-1 && idirection==1) {
      newval = gsl_rng_uniform(_r)*pthresh;
      _pvarspacer[imotif][0]=newval;
      for (int ivs=1; ivs<_lvarspacer[imotif]; ivs++) {
	_pvarspacer[imotif][ivs]=old_pvarspacer[ivs-1]*(1.0-newval);
      }      
    }
    if (iside==-1 && idirection==-1) {
      oldval = old_pvarspacer[0];
      for (int ivs=0; ivs<_lvarspacer[imotif]; ivs++) {
	_pvarspacer[imotif][ivs]=old_pvarspacer[ivs+1]/(1.0-oldval);
      }      
    }
    for (int ivs=_lvarspacer[imotif]; ivs<_lmaxvarspacer; ivs++) {
      _pvarspacer[imotif][ivs]=0;
    }      


    double oldlogltot = _logltot;
    double* old_loglseq = new double [_nseqs];
    int* old_alloc_imotif = new int [_nseqs];
    for (int iseq=0; iseq<_nseqs; iseq++) {
      old_loglseq[iseq] = _logl_seq_by_motif[iseq][imotif];
      old_alloc_imotif[iseq] = _alloc_imotif[iseq];
      for (int t=0; t<_lseq; t++) {
	_tmp_ratio_seqt[iseq][t] = _ratio_seqt[iseq][imotif][t];
      }
    }
    this->Compute_ratio_seqt_for_motif(imotif);
    this->Compute_logl_seq_for_motif(imotif);
    this->Compute_logltot_and_allocimotif();

    double paccept = exp(_logltot - oldlogltot); 
    paccept *= gsl_ran_dirichlet_pdf(_lvarspacer[imotif], alpha, _pvarspacer[imotif])/gsl_ran_dirichlet_pdf(old_lvarspacer, alpha, old_pvarspacer);
    if (idirection==1) {
      paccept *= pow(1.0-newval,old_lvarspacer-1)/(1.0/pthresh);
    } else {
      paccept *= (1.0/pthresh)/pow(1.0-oldval,_lvarspacer[imotif]-1);
    }
    //cout << "paccept " << paccept << endl;
    if (gsl_ran_bernoulli(_r, paccept)==1) {
      //cout << "accepted" << endl;
    } else {
      //cout << "rejected" << endl; 
      _logltot = oldlogltot;
      for (int ivs=0; ivs<_lmaxvarspacer; ivs++) {
	_pvarspacer[imotif][ivs] = old_pvarspacer[ivs];
      }
      _lspacer[imotif] = old_lspacer;
      _lvarspacer[imotif] = old_lvarspacer;
      for (int iseq=0; iseq<_nseqs; iseq++) {
	_logl_seq_by_motif[iseq][imotif] = old_loglseq[iseq];
	_alloc_imotif[iseq] = old_alloc_imotif[iseq];
	for (int t=0; t<_lseq; t++) {
	  _ratio_seqt[iseq][imotif][t]=_tmp_ratio_seqt[iseq][t];
	}
      }
    }
    delete [] old_pvarspacer;
    delete [] alpha;

    delete [] old_loglseq;
    delete [] old_alloc_imotif;
  }
}

void Mmodel::Update_Box2isoptional()
{
  if (_box2oblig) {
    return;
  }
  int imotif = gsl_rng_uniform_int(_r,_nmotifs);
  double pthresh = 0.9;

  //  cout << "Mmodel::Update_Switchvarspacer() imotif "<< imotif << " iside " << iside << endl;
  if ((_pbox2[imotif]>=pthresh) || (_box2isoptional[imotif]==0)) {
    double oldpbox2 = _pbox2[imotif];
    int typemove=1;
    if (_box2isoptional[imotif]==1) {
      typemove=0;
    }
    if (typemove==0) {
      _pbox2[imotif] = 1;
    } else {
      _pbox2[imotif] = pthresh + gsl_rng_uniform(_r)*(1-pthresh);      
    }
    //cout << "attempting move " << oldpbox2 << " -> " << _pbox2[imotif] << endl;
    double oldlogltot = _logltot;
    double* old_loglseq = new double [_nseqs];
    int* old_alloc_imotif = new int [_nseqs];
    for (int iseq=0; iseq<_nseqs; iseq++) {
      old_loglseq[iseq] = _logl_seq_by_motif[iseq][imotif];
      old_alloc_imotif[iseq] = _alloc_imotif[iseq];
      for (int t=0; t<_lseq; t++) {
	_tmp_ratio_seqt[iseq][t] = _ratio_seqt[iseq][imotif][t];
      }
    }
    this->Compute_ratio_seqt_for_motif(imotif);
    this->Compute_logl_seq_for_motif(imotif);
    this->Compute_logltot_and_allocimotif();

    double paccept = exp(_logltot - oldlogltot);
    if (typemove==0) {
      paccept *= (1-_minpbox2)/(1-pthresh);
    } else {
      paccept *= (1-pthresh)/(1-_minpbox2);
    }
    //    cout << "paccept " << paccept << endl;
    if (gsl_ran_bernoulli(_r, paccept)==1) {
      _box2isoptional[imotif] = typemove;
      //cout << "accepted" << endl;
    } else {
      //cout << "rejected" << endl; 
      _logltot = oldlogltot;
      _pbox2[imotif] = oldpbox2;

      for (int iseq=0; iseq<_nseqs; iseq++) {
	_logl_seq_by_motif[iseq][imotif] = old_loglseq[iseq];
	_alloc_imotif[iseq] = old_alloc_imotif[iseq];
	for (int t=0; t<_lseq; t++) {
	  _ratio_seqt[iseq][imotif][t]=_tmp_ratio_seqt[iseq][t];
	}
      }
    }
    delete [] old_loglseq;
    delete [] old_alloc_imotif;
  }
}

void Mmodel::Compute_logl_seq_for_motif(int imotif)
{
  for (int iseq=0; iseq<_nseqs; iseq++) {
    double psum = 0;
    for (int tbox1=0; tbox1<_lseq; tbox1++) {
      int tbox1refpos = tbox1refpos_from_tbox1(_ref_posbox1[imotif],tbox1);
      if (tbox1refpos>=_lmaxbox1 && tbox1refpos<_lseq-_maxdistfrom3end) {
	psum += _ratio_seqt[iseq][imotif][tbox1]*_p_pos_motif[tbox1refpos];
      }
    }
    _logl_seq_by_motif[iseq][imotif] = log(psum) + _logl_cumsum_background_seq[iseq][_lseq-1]; 
    //cout << "iseq " << iseq << " imotif " << imotif << " " << _logl_seq_by_motif[iseq][imotif] << endl;
  }  
}

void Mmodel::Compute_logl_seq_by_motif()
{
  for (int imotif=0; imotif<_nmotifs; imotif++) {
    this->Compute_logl_seq_for_motif(imotif);
  }
}

void Mmodel::Alloc_tbox1refpos_and_varspacer(int iseq, int imotif)
{
  double vsum = 0;
  double* tt = new double [_lseq];
  for (int tbox1refpos=_lmaxbox1; tbox1refpos<_lseq-_maxdistfrom3end; tbox1refpos++) {
    int tbox1 = tbox1_from_tbox1refpos(_ref_posbox1[imotif],tbox1refpos);
    tt[tbox1refpos] = _ratio_seqt[iseq][imotif][tbox1]*_p_pos_motif[tbox1refpos];
    vsum += tt[tbox1refpos];
  }  
  _alloc_tbox1refpos[iseq]=sampleProbVect(_lseq-_maxdistfrom3end-_lmaxbox1, tt+_lmaxbox1, vsum)+_lmaxbox1;
  int ibox1=tbox1_from_tbox1refpos(_ref_posbox1[imotif],_alloc_tbox1refpos[iseq]);
  
  double* xx = new double [_lvarspacer[imotif]+1];
  double xbox1 = exp(_logl_ratio_box1[iseq][imotif][ibox1]);
  xx[_lvarspacer[imotif]] = xbox1*(1-_pbox2[imotif]);
  for (int ivs=0; ivs<_lvarspacer[imotif]; ivs++) {
    xx[ivs] = xbox1*_pbox2[imotif]*_pvarspacer[imotif][ivs];
    int ibox2 = ibox1-_lspacer[imotif]-1-ivs;
    if (ibox2>=0) {
      xx[ivs] *= exp(_logl_ratio_box2[iseq][imotif][ibox2]);
    }
  }
  vsum=0;
  for (int i=0; i<=_lvarspacer[imotif]; i++) {
    vsum += xx[i];
  }
  _alloc_varspacer[iseq]=sampleProbVect(_lvarspacer[imotif]+1, xx, vsum);
  if (_alloc_varspacer[iseq]>=_lvarspacer[imotif]) {
    _alloc_varspacer[iseq]=-1;
  }
  delete [] xx;
  delete [] tt;
  //cout << "imotif " << imotif << " _alloc_tbox1refpos " << _alloc_tbox1refpos[iseq] << " _alloc_varspacer " << _alloc_varspacer[iseq] << endl;
}

void Mmodel::Update_refpos()
{
  int imotif = gsl_rng_uniform_int(_r,_nmotifs);
  int refposdiff = -1+2*gsl_ran_bernoulli(_r, 0.5);
  //cout << "Mmodel::Update_refpos()" << "  imotif " << imotif << " refposdiff " << refposdiff << endl;
  int oldrefpos = _ref_posbox1[imotif];
  int newrefpos = oldrefpos+refposdiff;
  if (newrefpos<_lminbox1-1 || newrefpos>=_lbox1[imotif]) {
    //cout << "  reject (out of range)" << endl;
    return;
  } else {
    //cout << "  attempting move " << oldrefpos << " -> " << newrefpos << endl;
    double* old_loglseq = new double [_nseqs];
    int* old_alloc_imotif = new int [_nseqs];
    double oldlogltot = _logltot;
    _ref_posbox1[imotif] = newrefpos;
    for (int iseq=0; iseq<_nseqs; iseq++) {
      old_loglseq[iseq] = _logl_seq_by_motif[iseq][imotif];
      old_alloc_imotif[iseq] = _alloc_imotif[iseq];
    }
    this->Compute_logl_seq_for_motif(imotif);
    //this->Recompute_logl_seq_for_motif_newrefpos(imotif,oldrefpos,newrefpos);
    this->Compute_logltot_and_allocimotif();
    double paccept = exp(_logltot - oldlogltot);
    //cout << "paccept " << paccept << endl;
    if (gsl_ran_bernoulli(_r, paccept)==1) {
      //cout << "accepted" << endl;
    } else {
      //cout << "rejected" << endl; 
      _logltot = oldlogltot;
      _ref_posbox1[imotif] = oldrefpos;
      for (int iseq=0; iseq<_nseqs; iseq++) {
	_logl_seq_by_motif[iseq][imotif] = old_loglseq[iseq];
	_alloc_imotif[iseq] = old_alloc_imotif[iseq];
      }
    }
    delete [] old_loglseq;
    delete [] old_alloc_imotif;
  }
}

void Mmodel::Update_Box2Width(Sequences* seqs, int imotif)
{
  int iside = -1+2*gsl_ran_bernoulli(_r, 0.5);
  int increase = -1+2*gsl_ran_bernoulli(_r, 0.5);

  //  cout << "Update_Box2Width imotif " << imotif << " iside " << iside << " increase " << increase << endl; 
  if ((increase==1 && _lbox2[imotif]==_lmaxbox2) 
      || (increase==-1 && _lbox2[imotif]==_lminbox2)) {
    return;
  } if (iside==1 && increase==1 && _lspacer[imotif]==_lminspacer) {
    return;
  } else if (iside==1 && increase==-1 && _lspacer[imotif]==_lmaxspacer) {
    return;
  }

  int tshift = 0;
  if (iside==1 && increase==-1) {
    tshift = -_lspacer[imotif] - 1;
  }
  if (iside==1 && increase==1) {
    tshift = -_lspacer[imotif];
  }
  if (iside==-1 && increase==-1) {
    tshift = -_lspacer[imotif] - 1 - _lbox2[imotif] + 1;
  }
  if (iside==-1 && increase==1) {
    tshift = -_lspacer[imotif] - 1 - _lbox2[imotif];
  }

  int* count_nt = new int [4];
  double* alpha = new double [4];

  double logl_background = 0;
  for (int ib=0; ib<4; ib++) {    
    count_nt[ib]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {    
    if (_alloc_imotif[iseq]==imotif) {
      if (_alloc_varspacer[iseq] != -1) {
	int* tseqM0 = *(seqs->sequencesMr(0)+iseq);
	int t =  tbox1_from_tbox1refpos(_ref_posbox1[imotif], _alloc_tbox1refpos[iseq]) + tshift - _alloc_varspacer[iseq];
	if (t>=0) {
	  count_nt[tseqM0[t]]++;
	  logl_background += _logl_cumsum_background_seq[iseq][t];
	  if (t>0) {
	    logl_background -= _logl_cumsum_background_seq[iseq][t-1];
	  }
	}
      }
    }
  }
  double tsum = 0;
  for (int ib=0; ib<4; ib++) {    
    tsum += count_nt[ib];
  }  

//   cout << "count_nt ";
//   for (int ib=0; ib<4; ib++) {    
//     cout << " " << count_nt[ib];
//   }  
//   cout << " sum " << tsum << " logl_background " << logl_background << endl;

  double logl_motif = gsl_sf_lngamma(4.0*_alpha_dirich_nuc) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum);
  for (int ib=0; ib<4; ib++) {    
    logl_motif += gsl_sf_lngamma(_alpha_dirich_nuc+(double)count_nt[ib]) - gsl_sf_lngamma(_alpha_dirich_nuc);
  }  
  //cout << "logl_motif " << logl_motif << endl;
  double paccept = exp(increase*(logl_motif - logl_background));
  //cout << "paccept " << paccept << endl;
  if (gsl_ran_bernoulli(_r, paccept)==1) {
    //cout << "accepted" << endl;
    if (increase==-1) {
      if (iside==1) {
	_lspacer[imotif]++; 
      } else if (iside==-1) {
	for (int ipos=1; ipos<_lbox2[imotif]; ipos++) {
	  for (int ib=0; ib<4; ib++) {
	    _log_pmatbox2[imotif][ipos-1][ib]=_log_pmatbox2[imotif][ipos][ib];
	  }
	}
	_track_posbox2[imotif]++;
      }
      _lbox2[imotif]--;
    } else if (increase==1) {
      if (iside==-1) {
	for (int ipos=_lbox2[imotif]-1; ipos>=0; ipos--) {
	  for (int ib=0; ib<4; ib++) {
	    _log_pmatbox2[imotif][ipos+1][ib]=_log_pmatbox2[imotif][ipos][ib];
	  }
	}	
	_track_posbox2[imotif]--;
      } else {
      }
      int tpos = 0;
      if (iside==1) {
	tpos = _lbox2[imotif];
      }
      for (int ib=0; ib<4; ib++) {
	alpha[ib] = _alpha_dirich_nuc+(double)count_nt[ib];
      }
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox2[imotif][tpos]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox2[imotif][tpos][ib]=log(_log_pmatbox2[imotif][tpos][ib]);
      }
      _lbox2[imotif]++;
      if (iside==1) {
	_lspacer[imotif]--;
      }
    }
  }

  delete [] alpha;
  delete [] count_nt;
}

void Mmodel::Update_Box1Width(Sequences* seqs, int imotif)
{
  int iside = -1+2*gsl_ran_bernoulli(_r, 0.5);
  int increase = -1+2*gsl_ran_bernoulli(_r, 0.5);

  //  cout << "Update_Box1Width imotif " << imotif << " iside " << iside << " increase " << increase << endl; 
  if ((increase==1 && _lbox1[imotif]==_lmaxbox1) 
      || (increase==-1 && _lbox1[imotif]==_lminbox1)) {
    return;
  } if (iside==-1 && increase==1 && _lspacer[imotif]==_lminspacer) {
    return;
  } else if (iside==-1 && increase==-1 && _lspacer[imotif]==_lmaxspacer) {
    return;
  } else if (increase==-1 && iside==-1 && _ref_posbox1[imotif]==_lminbox1-1) {
    return;
  } else if (increase==-1 && iside==1 && _ref_posbox1[imotif]==_lbox1[imotif]-1) {
    return;
  }

  int tshift = 0;
  if (iside==1 && increase==-1) {
    tshift = _lbox1[imotif]-1;
  }
  if (iside==1 && increase==1) {
    tshift = _lbox1[imotif];
  }
  if (iside==-1 && increase==-1) {
    tshift = 0;
  }
  if (iside==-1 && increase==1) {
    tshift = -1;
  }

  int* count_nt = new int [4];
  double* alpha = new double [4];

  double logl_background = 0;
  for (int ib=0; ib<4; ib++) {    
    count_nt[ib]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {    
    if (_alloc_imotif[iseq]==imotif) {
      int* tseqM0 = *(seqs->sequencesMr(0)+iseq);
      int t =  tbox1_from_tbox1refpos(_ref_posbox1[imotif], _alloc_tbox1refpos[iseq]) + tshift;
      if (t>=0 && t<_lseq) {
	count_nt[tseqM0[t]]++;
	logl_background += _logl_cumsum_background_seq[iseq][t];
	if (t>0) {
	  logl_background -= _logl_cumsum_background_seq[iseq][t-1];
	}
      }
    }
  }
  double tsum = 0;
  for (int ib=0; ib<4; ib++) {    
    tsum += count_nt[ib];
  }  

//    cout << "count_nt ";
//    for (int ib=0; ib<4; ib++) {    
//      cout << " " << count_nt[ib];
//    }  
//    cout << " sum " << tsum << " logl_background " << logl_background << endl;

  double logl_motif = gsl_sf_lngamma(4.0*_alpha_dirich_nuc) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum);
  for (int ib=0; ib<4; ib++) {    
    logl_motif += gsl_sf_lngamma(_alpha_dirich_nuc+(double)count_nt[ib]) - gsl_sf_lngamma(_alpha_dirich_nuc);
  }  
  // cout << "logl_motif " << logl_motif << endl;
  double paccept = exp(increase*(logl_motif - logl_background));
  paccept*=(1.0/(double)(_lbox1[imotif]-(_lminbox1-1)+increase))/(1.0/(double)(_lbox1[imotif]-(_lminbox1-1)));
  // this term is for _ref_posbox1;
  // cout << "paccept " << paccept << endl;
  if (gsl_ran_bernoulli(_r, paccept)==1) {
    //cout << "accepted" << endl;
    if (increase==-1) {
      if (iside==-1) {
	for (int ipos=1; ipos<_lbox1[imotif]; ipos++) {
	  for (int ib=0; ib<4; ib++) {
	    _log_pmatbox1[imotif][ipos-1][ib]=_log_pmatbox1[imotif][ipos][ib];
	  }
	}
	_ref_posbox1[imotif]--;
	_lspacer[imotif]++;     
	_track_posbox1[imotif]++;
      } else {
      }
      _lbox1[imotif]--;
    } else if (increase==1) {
      if (iside==-1) {
	for (int ipos=_lbox1[imotif]-1; ipos>=0; ipos--) {
	  for (int ib=0; ib<4; ib++) {
	    _log_pmatbox1[imotif][ipos+1][ib]=_log_pmatbox1[imotif][ipos][ib];
	  }
	}	
	_track_posbox1[imotif]--;
      } else {
      }
      int tpos = 0;
      if (iside==1) {
	tpos = _lbox1[imotif];
      }
      for (int ib=0; ib<4; ib++) {
	alpha[ib] = _alpha_dirich_nuc+(double)count_nt[ib];
      }
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox1[imotif][tpos]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox1[imotif][tpos][ib]=log(_log_pmatbox1[imotif][tpos][ib]);
      }
      if (iside==-1) {
	_ref_posbox1[imotif]++;
	_lspacer[imotif]--;
      }
      _lbox1[imotif]++;
    }
  }

  delete [] alpha;
  delete [] count_nt;
}

void Mmodel::Shift_Box1Width(Sequences* seqs, int imotif)
{
  if (_lbox1[imotif]!=_lmaxbox1) {
    return;
  }

  int iside = -1+2*gsl_ran_bernoulli(_r, 0.5);
  //try to add a position at side iside
  //cout << "Shift_Box1Width " << iside << endl;

  if (iside==-1 && _lspacer[imotif]==_lminspacer) {
    return;
  } else if (iside==1 && _lspacer[imotif]==_lmaxspacer) {
    return;
  } else if (iside==1 && _ref_posbox1[imotif]==_lminbox1-1) {
    return;
  } else if (iside==-1 && _ref_posbox1[imotif]==_lbox1[imotif]-1) {
    return;
  }

  int tshift_new = 0;
  int tshift_old = 0;
  if (iside==1) {
    tshift_new = _lbox1[imotif];
    tshift_old = 0;
  }
  if (iside==-1) {
    tshift_new = -1;
    tshift_old = _lbox1[imotif]-1;
  }

  int* count_nt_new = new int [4];
  int* count_nt_old = new int [4];
  double* alpha = new double [4];

  double logl_background_new = 0;
  double logl_background_old = 0;

  for (int ib=0; ib<4; ib++) {    
    count_nt_new[ib]=0;
    count_nt_old[ib]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {    
    if (_alloc_imotif[iseq]==imotif) {
      int* tseqM0 = *(seqs->sequencesMr(0)+iseq);
      int tnew =  tbox1_from_tbox1refpos(_ref_posbox1[imotif], _alloc_tbox1refpos[iseq]) + tshift_new;
      int told =  tbox1_from_tbox1refpos(_ref_posbox1[imotif], _alloc_tbox1refpos[iseq]) + tshift_old;
      if (tnew>=0 && tnew<_lseq) {
	count_nt_new[tseqM0[tnew]]++;
	logl_background_new += _logl_cumsum_background_seq[iseq][tnew];
	if (tnew>0) {
	  logl_background_new -= _logl_cumsum_background_seq[iseq][tnew-1];
	}
      }
      if (told>=0 && told<_lseq) {
	count_nt_old[tseqM0[told]]++;
	logl_background_old += _logl_cumsum_background_seq[iseq][told];
	if (told>0) {
	  logl_background_old -= _logl_cumsum_background_seq[iseq][told-1];
	}
      }
    }
  }
  double tsum_new = 0;
  double tsum_old = 0;
  for (int ib=0; ib<4; ib++) {    
    tsum_new += count_nt_new[ib];
    tsum_old += count_nt_old[ib];
  }  
  double logl_motif_new = gsl_sf_lngamma(4.0*_alpha_dirich_nuc) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum_new);
  for (int ib=0; ib<4; ib++) {    
    logl_motif_new += gsl_sf_lngamma(_alpha_dirich_nuc+(double)count_nt_new[ib]) - gsl_sf_lngamma(_alpha_dirich_nuc);
  }  
  double logl_motif_old = gsl_sf_lngamma(4.0*_alpha_dirich_nuc) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum_old);
  for (int ib=0; ib<4; ib++) {    
    logl_motif_old += gsl_sf_lngamma(_alpha_dirich_nuc+(double)count_nt_old[ib]) - gsl_sf_lngamma(_alpha_dirich_nuc);
  }  

  //cout << "count_nt " << endl;
  //for (int ib=0; ib<4; ib++) {    
  //  cout << count_nt_old[ib] << " -> " << count_nt_new[ib] << endl;
  //}  
  
  //cout << "logl_motif " << logl_motif_new << " " << logl_motif_old << endl;
  //cout << "logl_background " << logl_background_new << " " << logl_background_old << endl;
  double paccept = exp((logl_motif_new - logl_background_new)-(logl_motif_old - logl_background_old));
  //cout << "paccept " << paccept << endl;
  if (gsl_ran_bernoulli(_r, paccept)==1) {
    //cout << "accepted" << endl;
    for (int ib=0; ib<4; ib++) {
      alpha[ib] = _alpha_dirich_nuc+(double)count_nt_new[ib];
    }
    if (iside==-1) {
      for (int ipos=_lbox1[imotif]-1; ipos>=1; ipos--) {
	for (int ib=0; ib<4; ib++) {
	  _log_pmatbox1[imotif][ipos][ib]=_log_pmatbox1[imotif][ipos-1][ib];
	}
      }
      _ref_posbox1[imotif]++;
      _lspacer[imotif]--;     
      _track_posbox1[imotif]--;
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox1[imotif][0]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox1[imotif][0][ib]=log(_log_pmatbox1[imotif][0][ib]);
      }
    } else if (iside==1) {
      for (int ipos=0; ipos<_lbox1[imotif]-1; ipos++) {
	for (int ib=0; ib<4; ib++) {
	  _log_pmatbox1[imotif][ipos][ib]=_log_pmatbox1[imotif][ipos+1][ib];
	}
      }
      _ref_posbox1[imotif]--;
      _lspacer[imotif]++;     
      _track_posbox1[imotif]++;
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox1[imotif][_lbox1[imotif]-1]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox1[imotif][_lbox1[imotif]-1][ib]=log(_log_pmatbox1[imotif][_lbox1[imotif]-1][ib]);
      }
    }
  }

  delete [] alpha;
  delete [] count_nt_new;
  delete [] count_nt_old;
}

void Mmodel::Shift_Box2Width(Sequences* seqs, int imotif)
{
  if (_lbox2[imotif]!=_lmaxbox2) {
    return;
  }

  int iside = -1+2*gsl_ran_bernoulli(_r, 0.5);
  //try to add a position at side iside
  //cout << "Shift_Box2Width " << iside << endl;

  if (iside==-1 && _lspacer[imotif]==_lmaxspacer) {
    return;
  } else if (iside==1 && _lspacer[imotif]==_lminspacer) {
    return;
  }

  int tshift_new = 0;
  int tshift_old = 0;
  if (iside==1) {
    tshift_new = -_lspacer[imotif];
    tshift_old = -_lspacer[imotif] - _lbox2[imotif];
  }
  if (iside==-1) {
    tshift_new = -_lspacer[imotif] - _lbox2[imotif] - 1;
    tshift_old = -_lspacer[imotif] - 1;
  }

  int* count_nt_new = new int [4];
  int* count_nt_old = new int [4];
  double* alpha = new double [4];

  double logl_background_new = 0;
  double logl_background_old = 0;

  for (int ib=0; ib<4; ib++) {    
    count_nt_new[ib]=0;
    count_nt_old[ib]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {    
    if (_alloc_imotif[iseq]==imotif) {
      int* tseqM0 = *(seqs->sequencesMr(0)+iseq);
      int tnew =  tbox1_from_tbox1refpos(_ref_posbox1[imotif], _alloc_tbox1refpos[iseq]) + tshift_new - _alloc_varspacer[iseq];
      int told =  tbox1_from_tbox1refpos(_ref_posbox1[imotif], _alloc_tbox1refpos[iseq]) + tshift_old - _alloc_varspacer[iseq];
      if (tnew>=0 && tnew<_lseq) {
	count_nt_new[tseqM0[tnew]]++;
	logl_background_new += _logl_cumsum_background_seq[iseq][tnew];
	if (tnew>0) {
	  logl_background_new -= _logl_cumsum_background_seq[iseq][tnew-1];
	}
      }
      if (told>=0 && told<_lseq) {
	count_nt_old[tseqM0[told]]++;
	logl_background_old += _logl_cumsum_background_seq[iseq][told];
	if (told>0) {
	  logl_background_old -= _logl_cumsum_background_seq[iseq][told-1];
	}
      }
    }
  }
  double tsum_new = 0;
  double tsum_old = 0;
  for (int ib=0; ib<4; ib++) {    
    tsum_new += count_nt_new[ib];
    tsum_old += count_nt_old[ib];
  }  
  double logl_motif_new = gsl_sf_lngamma(4.0*_alpha_dirich_nuc) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum_new);
  for (int ib=0; ib<4; ib++) {    
    logl_motif_new += gsl_sf_lngamma(_alpha_dirich_nuc+(double)count_nt_new[ib]) - gsl_sf_lngamma(_alpha_dirich_nuc);
  }  
  double logl_motif_old = gsl_sf_lngamma(4.0*_alpha_dirich_nuc) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum_old);
  for (int ib=0; ib<4; ib++) {    
    logl_motif_old += gsl_sf_lngamma(_alpha_dirich_nuc+(double)count_nt_old[ib]) - gsl_sf_lngamma(_alpha_dirich_nuc);
  }  

  //cout << "count_nt " << endl;
  //for (int ib=0; ib<4; ib++) {    
  //  cout << count_nt_old[ib] << " -> " << count_nt_new[ib] << endl;
  //}  
  
  //cout << "logl_motif " << logl_motif_new << " " << logl_motif_old << endl;
  //cout << "logl_background " << logl_background_new << " " << logl_background_old << endl;
  double paccept = exp((logl_motif_new - logl_background_new)-(logl_motif_old - logl_background_old));
  //cout << "paccept " << paccept << endl;
  if (gsl_ran_bernoulli(_r, paccept)==1) {
    //cout << "accepted" << endl;
    for (int ib=0; ib<4; ib++) {
      alpha[ib] = _alpha_dirich_nuc+(double)count_nt_new[ib];
    }
    if (iside==-1) {
      for (int ipos=_lbox2[imotif]-1; ipos>=1; ipos--) {
	for (int ib=0; ib<4; ib++) {
	  _log_pmatbox2[imotif][ipos][ib]=_log_pmatbox2[imotif][ipos-1][ib];
	}
      }
      _lspacer[imotif]++;     
      _track_posbox2[imotif]--;
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox2[imotif][0]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox2[imotif][0][ib]=log(_log_pmatbox2[imotif][0][ib]);
      }
    } else if (iside==1) {
      for (int ipos=0; ipos<_lbox2[imotif]-1; ipos++) {
	for (int ib=0; ib<4; ib++) {
	  _log_pmatbox2[imotif][ipos][ib]=_log_pmatbox2[imotif][ipos+1][ib];
	}
      }
      _lspacer[imotif]--;     
      _track_posbox2[imotif]++;
      gsl_ran_dirichlet(_r, 4, alpha, _log_pmatbox2[imotif][_lbox2[imotif]-1]);
      for (int ib=0; ib<4; ib++) {
	_log_pmatbox2[imotif][_lbox2[imotif]-1][ib]=log(_log_pmatbox2[imotif][_lbox2[imotif]-1][ib]);
      }
    }
  }

  delete [] alpha;
  delete [] count_nt_new;
  delete [] count_nt_old;
}


void Mmodel::Compute_logltot_and_allocimotif()
{
  if (_tree==NULL) {
    double logltot = 0;
    double* tpmotif = new double [_dimlikcomput];
    for (int iseq=0; iseq<_nseqs; iseq++) {    
      double tmax = _logl_seq_by_motif[iseq][0];
      for (int imotif=1; imotif<_dimlikcomput; imotif++) {
	if (_logl_seq_by_motif[iseq][imotif]>tmax) {
	  tmax = _logl_seq_by_motif[iseq][imotif];
	}
      }
      double tpsum = 0;
      for (int imotif=0; imotif<_dimlikcomput; imotif++) {
	tpmotif[imotif] = _pmotifs[imotif]*exp(_logl_seq_by_motif[iseq][imotif]-tmax);
	tpsum += tpmotif[imotif];
      }    
      logltot += log(tpsum) + tmax;
      //cout << "tpmotif[imotif]";
      for (int imotif=0; imotif<_dimlikcomput; imotif++) {
	tpmotif[imotif] /= tpsum;
	//cout << " " << tpmotif[imotif];
      }
      _alloc_imotif[iseq]=sampleProbVect(_dimlikcomput,tpmotif);
    }
    //cout << "logltot " << logltot << endl;
    _logltot = logltot;
  } else {
    double logltot = _tree->ComputeLogTotAndAllocimotif(_epsilon_evolontree,_alpha_evolontree,_pmotifs,_pmotifs_ontree, _logl_seq_by_motif, _alloc_imotif);
    //cout << "logltot on tree " << logltot << endl;
    _logltot = logltot;    
  }
}


void Mmodel::Alloc_tbox1refpos_and_varspacer_allseq()
{
  for (int iseq=0; iseq<_nseqs; iseq++) {    
    if (_alloc_imotif[iseq]<_nmotifs) {
      this->Alloc_tbox1refpos_and_varspacer(iseq, _alloc_imotif[iseq]);
      //cout << " _alloc_imotif[iseq] " << iseq << " " << _alloc_imotif[iseq] << endl;
    } else {
      _alloc_tbox1refpos[iseq] = -1;
      _alloc_varspacer[iseq] = -1;
    }
  }
}

void Mmodel::printParams(ostream& os)
{
  os << "_nbins_p_pos_motif " << _nbins_p_pos_motif << endl;
  os << "_breakpoints_p_pos_motif";
  for (int t=0; t<_lseq; t++) {
    os << " " << _breakpoints_p_pos_motif[t];
  }
  os << endl;
  os << "_p_pos_motif";
  for (int t=0; t<_lseq; t++) {
    os << " " << _p_pos_motif[t];
  }
  os << endl;
  int codemax = (int)((pow(4,_rbackg+2)-4.0)/3.0);
  os << "_backgroundMr";
  for (int i=0; i<codemax; i++) {
    os << " " << exp(_log_backgroundMr[i]);
  }
  os << endl;
  if (_tree!=NULL) {
    os << "_pmotifs_ontree";
    for (int im=0; im<_dimlikcomput; im++) {
      os << " " << _pmotifs_ontree[im];
    }
    os << endl;
  }
  os << "_pmotifs";
  for (int im=0; im<_dimlikcomput; im++) {
    os << " " << _pmotifs[im];
  }
  os << endl;
  os << "_alpha_evolontree " << _alpha_evolontree << " _epsilon_evolontree " << _epsilon_evolontree << endl;
  for (int im=0; im<_nmotifs; im++) {
    os << " motif " << im << " pmotif_ontree ";
    if (_tree!=NULL) {
      os << _pmotifs_ontree[im];
    } else {
      os << "NA";
    }
    os << " pmotif " <<_pmotifs[im] << " lspacer " << _lspacer[im] << " lvarspacer " << _lvarspacer[im] << " _box2isoptional " << _box2isoptional[im] << " pbox2 " << _pbox2[im] << " _track_posbox1 " << _track_posbox1[im] << " _track_posbox2 " << _track_posbox2[im] << " _lbox1 " << _lbox1[im] << " _lbox2 " << _lbox2[im] << " _ref_posbox1 " << _ref_posbox1[im] << endl;
    os << "  _pvarspacer";
    for (int ivs=0; ivs<_lmaxvarspacer; ivs++) {
      os << " " << _pvarspacer[im][ivs];
    }
    os << endl;
    os << "  box1 " << "_lbox1 " << _lbox1[im] << " _ref_posbox1 " << _ref_posbox1[im] << endl;
    for (int ipos=0; ipos<_lbox1[im]; ipos++) {
      os << "   " << ipos << " _pmatbox1";
      for (int ib=0; ib<4; ib++) {
	os << " " << exp(_log_pmatbox1[im][ipos][ib]);
      }
      os << endl;
    }
    os << "  box2 " << "_lbox2 " << _lbox2[im] << endl;
    for (int ipos=0; ipos<_lbox2[im]; ipos++) {
      os << "   " << ipos << " _pmatbox2";
      for (int ib=0; ib<4; ib++) {
	os << " " << exp(_log_pmatbox2[im][ipos][ib]);
      }
      os << endl;
    }
    
  }  
}

void Mmodel::printAlloc(ostream& os, Sequences* seqs)
{
  os << "alloc";
  for (int iseq=0; iseq<_nseqs; iseq++) {
    double loglbox1; 
    double loglbox2;
    this->ComputeLoglRatioCurrentAlloc(&loglbox1, &loglbox2, iseq);
    os << "\t" << "s" << seqs->SeqId(iseq) << "\t" << _alloc_imotif[iseq] << "\t" << _alloc_tbox1refpos[iseq] << "\t" << _alloc_varspacer[iseq] << "\t" << floor(1000*loglbox1+0.5)/1000 << "\t" << floor(1000*loglbox2+0.5)/1000;
  }
  os << endl;
}

void Mmodel::ComputeLoglRatioCurrentAlloc(double* loglbox1, double* loglbox2, int iseq)
{
  if (_alloc_imotif[iseq]<_nmotifs) {
    int imotif = _alloc_imotif[iseq];
    int ibox1=tbox1_from_tbox1refpos(_ref_posbox1[imotif],_alloc_tbox1refpos[iseq]);
    *loglbox1=_logl_ratio_box1[iseq][imotif][ibox1];
    if (_alloc_varspacer[iseq]>=0) {
      int ibox2 = ibox1-_lspacer[imotif]-1-_alloc_varspacer[iseq];
      *loglbox2 = _logl_ratio_box2[iseq][imotif][ibox2];
    } else {
      *loglbox2 = 0;
    }
  } else {
    *loglbox1 = 0;
    *loglbox2 = 0;
  }  
}

void Mmodel::simulSeqM1(Sequences* seqs)
{
  if (_rbackg != 1) {
    cout << "error in Mmodel::simulSeqM1(Sequences* seqs)" << endl;
    exit(1);
  }
  double* p_backgroundM1 = new double [20];
  double* p_M0 = new double [4];
  double vsum;
  for (int ib=0; ib<20; ib++) {
    p_backgroundM1[ib] = exp(_log_backgroundMr[ib]);
  }
  vsum = 0;
  int ici = 0;
  while (ici<20) {
    vsum=0;
    for (int i=0; i<4; i++) {
      vsum += p_backgroundM1[ici+i];
    }
    for (int i=0; i<4; i++) {
      p_backgroundM1[ici+i] /= vsum;
    }    
    ici+=4;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    int* tseqM0 = *(seqs->sequencesMr(0)+iseq);
    for (int t=0; t<_lseq; t++) {
      tseqM0[t]=-1;
    }
    int im = _alloc_imotif[iseq];
    
    if (im<_nmotifs) {
      int b1tmin = tbox1_from_tbox1refpos(_ref_posbox1[im],
					  _alloc_tbox1refpos[iseq]);
      int b1tmax = b1tmin + _lbox1[im] - 1;
      int b1ipos = 0;
      for (int t = b1tmin; t <= b1tmax; t++) {
	if (t>=0 && t<_lseq) {
	  vsum=0;
	  for (int ib=0; ib<4; ib++) {
	    p_M0[ib] = exp(_log_pmatbox1[im][b1ipos][ib]);
	    vsum += p_M0[ib];
	  }
	  tseqM0[t] = this->sampleProbVect(4, p_M0, vsum);
	}
	b1ipos++;
      }
      
      if (_alloc_varspacer[iseq]>=0) { // otherwise no box2
	int b2tmax = b1tmin - 1 - _lspacer[im] - _alloc_varspacer[iseq];
	int b2tmin = b2tmax - _lbox2[im] + 1;
	int b2ipos = 0;
	for (int t = b2tmin; t<=b2tmax; t++) {
	  if (t>=0 && t<_lseq) {
	    vsum=0;
	    for (int ib=0; ib<4; ib++) {
	      p_M0[ib] = exp(_log_pmatbox2[im][b2ipos][ib]);
	      vsum += p_M0[ib];
	    }
	    tseqM0[t] = this->sampleProbVect(4, p_M0, vsum);
	  }
	  b2ipos++;
	}
      }
    }

    if (tseqM0[0]==-1) {
      int tsuff = 0;
      tseqM0[0]=this->sampleProbVect(4, p_backgroundM1+tsuff);
    }
    for (int t=1; t<_lseq; t++) {
      if (tseqM0[t]==-1) {
	int tsuff = 4 + tseqM0[t-1]*4;
	tseqM0[t]=this->sampleProbVect(4, p_backgroundM1+tsuff);
      }
    }    
    int* tseqM1 = *(seqs->sequencesMr(1)+iseq);
    tseqM1[0]=tseqM0[0];
    for (int t=1; t<_lseq; t++) {
      tseqM1[t] = 4 + tseqM0[t-1]*4 + tseqM0[t];
    }     
  } 
  delete [] p_backgroundM1;
  delete [] p_M0;
}

double Mmodel::ComputeLogl_BackgroundMrOnly(Sequences* seqs)
{
  int codemax = (int)((pow(4,_rbackg+2)-4.0)/3.0);

  double ret_logl = 0;
  double ret_logl_ml = 0;

  for (int ic=0; ic<codemax; ic++) {
    _count_nuc_backg[ic]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    int* tseqMr = *(seqs->sequencesMr(_rbackg)+iseq);
    for (int t=0; t<_lseq; t++) {
      _count_nuc_backg[tseqMr[t]]++;
    }
  }
  int codemaxinfr = (int)((pow(4,_rbackg+1)-4.0)/3.0);
  for (int ic=0; ic<codemaxinfr; ic++) {
    ret_logl += log(1.0/4.0)*(double)_count_nuc_backg[ic];
    ret_logl_ml += log(1.0/4.0)*(double)_count_nuc_backg[ic];
  }
  int ici=codemaxinfr;
  while (ici<codemax) {
    int tsum = 0;
    for (int i=ici+0; i<ici+4; i++) {
      tsum += _count_nuc_backg[i];
    }  
    ret_logl += gsl_sf_lngamma(4.0*_alpha_dirich_nuc) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum);
    for (int i=ici+0; i<ici+4; i++) {
      ret_logl += gsl_sf_lngamma(_alpha_dirich_nuc+(double)_count_nuc_backg[i]) - gsl_sf_lngamma(_alpha_dirich_nuc);
      ret_logl_ml += (double)_count_nuc_backg[i]*log((double)_count_nuc_backg[i]/(double)tsum);
    }
    ici+=4;
  }
  cout << "Model with background only (no motifs) Marginal Logl: " << ret_logl << " ML Logl: " << ret_logl_ml << endl;  
  return(ret_logl);
}

void Mmodel::Update_BackgroundMr(Sequences* seqs)
{
  double* alpha = new double [4];
  int codemax = (int)((pow(4,_rbackg+2)-4.0)/3.0);

  for (int ic=0; ic<codemax; ic++) {
    _count_nuc_backg[ic]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    int* tseqMr = *(seqs->sequencesMr(_rbackg)+iseq);
    int im = _alloc_imotif[iseq];
    for (int t=0; t<_lseq; t++) {
      _count_nuc_backg[tseqMr[t]]++;
    }
    if (im<_nmotifs) {
      int b1tmin = tbox1_from_tbox1refpos(_ref_posbox1[im],
					  _alloc_tbox1refpos[iseq]);
      int b1tmax = b1tmin + _lbox1[im] - 1;
      for (int t = b1tmin; t <= b1tmax; t++) {
	if (t>=0 && t<_lseq) {
	  _count_nuc_backg[tseqMr[t]]--;
	}
      }
      
      if (_alloc_varspacer[iseq]>=0) { // otherwise no box2
	int b2tmax = b1tmin - 1 - _lspacer[im] - _alloc_varspacer[iseq];
	int b2tmin = b2tmax - _lbox2[im] + 1;
	for (int t = b2tmin; t<=b2tmax; t++) {
	  if (t>=0 && t<_lseq) {
	    _count_nuc_backg[tseqMr[t]]--;
	  }
	}
      }
    }
  }

  int codemaxinfr = (int)((pow(4,_rbackg+1)-4.0)/3.0);
  for (int ic=0; ic<codemaxinfr; ic++) {
    _log_backgroundMr[ic] = log(1.0/4.0);
  }
  int ici=codemaxinfr;
  while (ici<codemax) {
    //cout << "counts ici=" << ici;
    for (int i=ici+0; i<ici+4; i++) {
      alpha[i-ici] = _alpha_dirich_nuc + (double)_count_nuc_backg[i];
      //cout << " " << _count_nuc_backg[i]; 
    }  
    //cout << endl;
    gsl_ran_dirichlet(_r, 4, alpha, _log_backgroundMr+ici);
    //cout << "pobs:";
    for (int i=ici+0; i<ici+4; i++) {

      _log_backgroundMr[i] = log(_log_backgroundMr[i]);
      //cout << " " << _log_backgroundMr[i]; 
    }      
    //cout << endl;
    ici+=4;
  }
  delete [] alpha;
}

void Mmodel::heuristic_split(Sequences* seqs) 
{
  if (!_allownomotif) {
    return;
  }

  int* count_motifs = new int [_dimlikcomput];
  for (int im=0; im<_dimlikcomput; im++) {
    count_motifs[im]=0;
  }  
  for (int iseq=0; iseq<_nseqs; iseq++) {
    count_motifs[_alloc_imotif[iseq]]++;
  }    
  list<pair<int,int> > mylist;
  for (int im=0; im<_nmotifs; im++) {
    mylist.push_back(pair<int, int>(count_motifs[im],im));
  }
  mylist.sort();
  mylist.reverse();
  delete [] count_motifs;

  list<pair<int,int> >::iterator it;
  for (it=mylist.begin(); it!=mylist.end(); it++) {
    //cout << it->first << " " << it->second << endl;
  }
  
  it=mylist.end();
  it--;
  if (it->first > 0.001*(double)_nseqs) {
    //cout << "no heuristic split" << endl;
    return;
  } else {
    cout << "heuristic split" << endl;
  }
  //cout << "last count " << it->first << endl;
  
  double* vectp = new double [_nmotifs];
  double tsum = 0;
  for (int i=0; i<_nmotifs; i++) {
    vectp[i] = 1.0/(double)(i+1);
    tsum += vectp[i];
  }

  int isplit = sampleProbVect(_nmotifs, vectp, tsum);
  int iremove = _nmotifs - 1;
  if (iremove==isplit) {
    iremove--;
  }
  int im_split = 0;
  int im_remove = 0;
  int k=0;
  for (it=mylist.begin(); it!=mylist.end(); it++) {
    if (k==isplit) {
      im_split = it->second;
    }
    if (k==iremove) {
      im_remove = it->second;
    }
    k++;
  }
  //cout << "isplit " << isplit << " iremove " << iremove << endl;
  //cout << "im_split " << im_split << " im_remove " << im_remove << endl;
  
  _lvarspacer[im_remove] = _lvarspacer[im_split];
  _pmotifs[im_split] /= 2.0;
  _pmotifs[im_remove] = _pmotifs[im_split];
  if (_tree != NULL) {
    _pmotifs_ontree[im_split] /= 2.0;
    _pmotifs_ontree[im_remove] = _pmotifs_ontree[im_split];
  }
  _lbox1[im_remove] = _lbox1[im_split];
  _lbox2[im_remove] = _lbox2[im_split];
  _track_posbox1[im_remove] = _track_posbox1[im_split];
  _track_posbox2[im_remove] = _track_posbox2[im_split];
  _ref_posbox1[im_remove] = _ref_posbox1[im_split];
  _box2isoptional[im_remove] = _box2isoptional[im_split];
  _pbox2[im_remove] = _pbox2[im_split];
  for (int ivs=0; ivs<_lmaxvarspacer; ivs++) {
    _pvarspacer[im_remove][ivs]=_pvarspacer[im_split][ivs];
  } 

  double psplit = gsl_rng_uniform(_r);
  for (int iseq=0; iseq<_nseqs; iseq++) {
    if (_alloc_imotif[iseq]==im_remove) {
      _alloc_imotif[iseq] = _nmotifs;
      _alloc_varspacer[iseq] = -1;
      _alloc_tbox1refpos[iseq] = -1;
    } else if (_alloc_imotif[iseq]==im_split) {
      if (gsl_ran_bernoulli(_r, psplit)==1) {
	_alloc_imotif[iseq] = im_remove;
      }
    }
  }    
  this->Update_pmatbox(seqs);
  this->Update_pvarspacer();
  delete [] vectp;
}

void Mmodel::Alloc_counts_subtree()
{

  int nsubtrees = _nseqs + (_nseqs-1);
  _count_nuc_matbox1_subtree = new int*** [nsubtrees];
  _count_nuc_matbox2_subtree = new int*** [nsubtrees];
  _count_varspacer_subtree = new int* [nsubtrees];
  _count_box2nobox2_subtree = new int* [nsubtrees];
  _same_count_as_subtree = new int [nsubtrees];
  _liktreesplit_subtree = new double [nsubtrees];
  _allowedsplit_subtree = new bool [nsubtrees];
  _loglikbackground_subtree = new double* [nsubtrees];
  for (int is=0; is<nsubtrees; is++) {
    _loglikbackground_subtree[is] = new double [2];
    _count_nuc_matbox1_subtree[is] = new int** [2];
    _count_nuc_matbox2_subtree[is] = new int** [2];
    _count_varspacer_subtree[is] = new int [_lmaxvarspacer];
    _count_box2nobox2_subtree[is] = new int [2];
    for (int im=0; im<2; im++) {
      _count_nuc_matbox1_subtree[is][im] = new int* [_lmaxbox1];
      _count_nuc_matbox2_subtree[is][im] = new int* [_lmaxbox2];
      for (int pos=0; pos<_lmaxbox1; pos++) {
	_count_nuc_matbox1_subtree[is][im][pos] = new int [4];
      }
      for (int pos=0; pos<_lmaxbox2; pos++) {
	_count_nuc_matbox2_subtree[is][im][pos] = new int [4];
      }
    }
  }
}


void Mmodel::mergeandsplit(Sequences* seqs) 
{
  if (_tree==NULL) {
    return;
  }
  int** compat_pairs = new int* [_dimlikcomput];
  double** weighted_compat_pairs = new double* [_dimlikcomput];
  for (int ic=0; ic<_dimlikcomput; ic++) {
    compat_pairs[ic] = new int [_dimlikcomput];
    weighted_compat_pairs[ic] = new double [_dimlikcomput];
  }
  int* count_alloc_imotif = new int [_nmotifs];
  for (int ic=0; ic<_nmotifs; ic++) {
    cout << ic << " box2isoptional " << _box2isoptional[ic] << " _lvarspacer " << _lvarspacer[ic] << endl;
  }

  //_tree->IndentifyCandidateForMergeAndSplit(compat_pairs);
  int* simul_nchanges_leaves;
  _tree->CompatPairsForMergeAndSplit(compat_pairs, simul_nchanges_leaves);

  
  for (int ic=0; ic<_nmotifs; ic++) {
    count_alloc_imotif[ic]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    if (_alloc_imotif[iseq]>=0) {
      count_alloc_imotif[_alloc_imotif[iseq]]++;
    }
  }
  cout << "count_alloc_imotif";
  for (int ic=0; ic<_nmotifs; ic++) {
    cout << " " << ic << ":" << count_alloc_imotif[ic];
  }
  cout << endl;

  for (int ic=0; ic<_nmotifs; ic++) {
    count_alloc_imotif[ic]=0;
  }
  for (int iseq=0; iseq<_nseqs; iseq++) {
    if (_alloc_imotif[iseq]>=0 && simul_nchanges_leaves[iseq]==0) {
      count_alloc_imotif[_alloc_imotif[iseq]]++;
    }
  }
  cout << "count_alloc_imotif no leaf changes";
  for (int ic=0; ic<_nmotifs; ic++) {
    cout << " " << ic << ":" << count_alloc_imotif[ic];
  }
  cout << endl;

  cout << "compat_pairs." << endl;
  for (int ic=0; ic<_nmotifs; ic++) {
    for (int jc=0; jc<_nmotifs; jc++) {
      if (ic==jc) {
	cout << " " << "x";
      } else if (compat_pairs[ic][jc]==0) {
	cout << " " << ".";
      } else {
	cout << " " << compat_pairs[ic][jc];
      }
    }  
    cout << endl;
  }  

  for (int ic=0; ic<_nmotifs; ic++) {
    for (int jc=0; jc<_nmotifs; jc++) {
      if (compat_pairs[ic][jc]==-1 || _lvarspacer[ic]!=_lvarspacer[jc] 
	  || _box2isoptional[ic]!=_box2isoptional[jc]) {
	weighted_compat_pairs[ic][jc]=0;
      } else {
	weighted_compat_pairs[ic][jc]=count_alloc_imotif[ic]+count_alloc_imotif[jc];
	//weighted_compat_pairs[ic][jc]=1.0+abs(_ref_posbox1[ic]+_lspacer[ic]-_ref_posbox1[jc]+_lspacer[jc]); //+abs(_lbox1[ic]-_lbox1[jc])+abs(_lbox2[ic]-_lbox2[jc]);
	//weighted_compat_pairs[ic][jc]=1.0/weighted_compat_pairs[ic][jc];
      }
    }
  }

  double wtot = 0;
  for (int ic=0; ic<_nmotifs; ic++) {
    for (int jc=0; jc<_nmotifs; jc++) {
      wtot += weighted_compat_pairs[ic][jc];
    }
  }  

  cout << "weighted compat_pairs. total weight " << wtot << endl;
  for (int ic=0; ic<_nmotifs; ic++) {
    for (int jc=0; jc<_nmotifs; jc++) {
      if (ic==jc) {
	cout << " " << "x";
      } else if (weighted_compat_pairs[ic][jc]==0) {
	cout << " " << ".";
      } else {
	cout << " " << weighted_compat_pairs[ic][jc];
      }
    }  
    cout << endl;
  }  

  if (0 && wtot>1e-10) {
    double rw = gsl_rng_uniform(_r)*wtot;
    double w=0;
    int rcomp1=-1;
    int rcomp2=-1;
    for (int ic=0; ic<_nmotifs; ic++) {
      for (int jc=0; jc<_nmotifs; jc++) {
	w += weighted_compat_pairs[ic][jc];
	if (rcomp1==-1 && rcomp2==-1 && w>=rw) {
	  rcomp1=ic;
	  rcomp2=jc;
	}
      }
    }
    int original_splitnode = compat_pairs[rcomp1][rcomp2];
    cout << "selected: " << rcomp1 << " x " << rcomp2 << " weight " << weighted_compat_pairs[rcomp1][rcomp2] << " original_splitnode " << original_splitnode << endl;
    cout << "lbox1 " << _lbox1[rcomp1] << " " << _lbox1[rcomp2] << " lbox2 " << _lbox2[rcomp1] << " " << _lbox2[rcomp2] << " pbox2 " << _pbox2[rcomp1] << " " << _pbox2[rcomp2] << endl;
    for (int iseq=0; iseq<_nseqs; iseq++) {
      int* tseq = *(seqs->sequencesMr(0)+iseq);
      if (_alloc_imotif[iseq]==rcomp1 || _alloc_imotif[iseq]==rcomp2) {
	_same_count_as_subtree[iseq]=iseq;
	for (int ivs=0; ivs<_lmaxvarspacer; ivs++) {
	  _count_varspacer_subtree[iseq][ivs]=0;
	}
	if (_alloc_varspacer[iseq]>=0) { // otherwise no box2
	  _count_box2nobox2_subtree[iseq][0]=1;
	  _count_box2nobox2_subtree[iseq][1]=0;
	  _count_varspacer_subtree[iseq][_alloc_varspacer[iseq]]++;
	} else {
	  _count_box2nobox2_subtree[iseq][0]=0;
	  _count_box2nobox2_subtree[iseq][1]=1;
	}
	this->InitCountsSubtree(iseq, rcomp1, 0, tseq);
	this->InitCountsSubtree(iseq, rcomp2, 1, tseq);		
      } else {
	_same_count_as_subtree[iseq]=-1;	
      }
    }

    _tree->recursSubtree(_lmaxbox1,_lmaxbox2,_lmaxvarspacer, rcomp1, rcomp2, 
			 _alpha_evolontree, _pmotifs_ontree,
			 _count_nuc_matbox1_subtree, 
			 _count_nuc_matbox2_subtree,
			 _count_varspacer_subtree,
			 _count_box2nobox2_subtree, 
			 _same_count_as_subtree,_allowedsplit_subtree,
			 _loglikbackground_subtree,
			 _liktreesplit_subtree, original_splitnode);
    int nsubtrees = _nseqs + (_nseqs-1);
    int* counts = new int [4];
    int bestsplit = -1;
    double* pselectallowedsplit = new double [nsubtrees];
    int* iallowedsplit = new int [nsubtrees];
    int nallowedsplit = 0;
    int newcount1;
    int newcount2;
    for (int is=0; is<nsubtrees; is++) {
      if (_allowedsplit_subtree[is]) {
	//cout << "subtree " << is << endl;
	double tloglikbox1rcomp1 = 0;
	double tloglikbox1rcomp2 = 0;
	double tloglikbox2rcomp1 = 0;
	double tloglikbox2rcomp2 = 0;
	double tloglikbackgrcomp1 = 0;
	double tloglikbackgrcomp2 = 0;
	double tloglikspacercomp1 = 0;
	double tloglikspacercomp2 = 0;
	double tloglikpbox2rcomp1 = 0;
	double tloglikpbox2rcomp2 = 0;
	if (is!=nsubtrees-1) {
	  if (_same_count_as_subtree[is]>=0) {
	    tloglikbox1rcomp1=this->ComputeLogLikForBoxCountVectComp1(_count_nuc_matbox1_subtree[_same_count_as_subtree[is]][0], _lbox1[rcomp1], false);
	    tloglikbox1rcomp2=this->ComputeLogLikForBoxCountVectComp2(_count_nuc_matbox1_subtree[_same_count_as_subtree[is]][1],_count_nuc_matbox1_subtree[_same_count_as_subtree[nsubtrees-1]][1], _lbox1[rcomp2], false);
	    tloglikbox2rcomp1=this->ComputeLogLikForBoxCountVectComp1(_count_nuc_matbox2_subtree[_same_count_as_subtree[is]][0], _lbox2[rcomp1], false);
	    tloglikbox2rcomp2=this->ComputeLogLikForBoxCountVectComp2(_count_nuc_matbox2_subtree[_same_count_as_subtree[is]][1],_count_nuc_matbox2_subtree[_same_count_as_subtree[nsubtrees-1]][1], _lbox2[rcomp2], false);
	    tloglikbackgrcomp1 += _loglikbackground_subtree[_same_count_as_subtree[is]][0];
	    tloglikbackgrcomp2 += _loglikbackground_subtree[_same_count_as_subtree[nsubtrees-1]][1] - _loglikbackground_subtree[_same_count_as_subtree[is]][1];

	    tloglikspacercomp1 += this->ComputeLogLikForSpacerCount(_count_varspacer_subtree[_same_count_as_subtree[is]], NULL, _lvarspacer[rcomp1], false);
	    tloglikspacercomp2 += this->ComputeLogLikForSpacerCount(_count_varspacer_subtree[_same_count_as_subtree[is]], _count_varspacer_subtree[_same_count_as_subtree[nsubtrees-1]], _lvarspacer[rcomp2], false);
	    if (_box2isoptional[rcomp1]==1) {
	      tloglikpbox2rcomp1 += this->ComputeLogLikForPbox2(_count_box2nobox2_subtree[_same_count_as_subtree[is]][0], _count_box2nobox2_subtree[_same_count_as_subtree[is]][1]);
	      tloglikpbox2rcomp2 += this->ComputeLogLikForPbox2(_count_box2nobox2_subtree[_same_count_as_subtree[nsubtrees-1]][0] - _count_box2nobox2_subtree[_same_count_as_subtree[is]][0], _count_box2nobox2_subtree[_same_count_as_subtree[nsubtrees-1]][1] - _count_box2nobox2_subtree[_same_count_as_subtree[is]][1]);
	    }
	  } else {
	    tloglikbox1rcomp1=this->ComputeLogLikForBoxCountVectComp1(NULL, _lbox1[rcomp1], true);
	    tloglikbox1rcomp2=this->ComputeLogLikForBoxCountVectComp2(NULL,_count_nuc_matbox1_subtree[_same_count_as_subtree[nsubtrees-1]][1], _lbox1[rcomp2], true);
	    tloglikbox2rcomp1=this->ComputeLogLikForBoxCountVectComp1(NULL, _lbox2[rcomp1], true);
	    tloglikbox2rcomp2=this->ComputeLogLikForBoxCountVectComp2(NULL,_count_nuc_matbox2_subtree[_same_count_as_subtree[nsubtrees-1]][1], _lbox2[rcomp2], true);
	    tloglikbackgrcomp2 += _loglikbackground_subtree[_same_count_as_subtree[nsubtrees-1]][1];
	    tloglikspacercomp1 += this->ComputeLogLikForSpacerCount(NULL, NULL, _lvarspacer[rcomp1], true);
	    tloglikspacercomp2 += this->ComputeLogLikForSpacerCount(NULL, _count_varspacer_subtree[_same_count_as_subtree[nsubtrees-1]], _lvarspacer[rcomp2], true);
	    if (_box2isoptional[rcomp1]==1) {
	      tloglikpbox2rcomp2 += this->ComputeLogLikForPbox2(_count_box2nobox2_subtree[_same_count_as_subtree[nsubtrees-1]][0], _count_box2nobox2_subtree[_same_count_as_subtree[nsubtrees-1]][1]);
	    }
	  }
	} else {
	  tloglikbox1rcomp2=this->ComputeLogLikForBoxCountVectComp1(_count_nuc_matbox1_subtree[_same_count_as_subtree[is]][1], _lbox1[rcomp2], false);
	  tloglikbox2rcomp2=this->ComputeLogLikForBoxCountVectComp1(_count_nuc_matbox2_subtree[_same_count_as_subtree[is]][1], _lbox2[rcomp2], false);
	  tloglikbackgrcomp2=_loglikbackground_subtree[_same_count_as_subtree[is]][1];
	  tloglikspacercomp2 += this->ComputeLogLikForSpacerCount(_count_varspacer_subtree[_same_count_as_subtree[is]], NULL, _lvarspacer[rcomp2], false);
	  if (_box2isoptional[rcomp1]==1) {
	    tloglikpbox2rcomp2 += this->ComputeLogLikForPbox2(_count_box2nobox2_subtree[_same_count_as_subtree[is]][0], _count_box2nobox2_subtree[_same_count_as_subtree[is]][1]);
	  }
	}
	double tlogliktree = _liktreesplit_subtree[is];
	double tsum  = tloglikbox1rcomp1+tloglikbox1rcomp2+tloglikbox2rcomp1+tloglikbox2rcomp2-tloglikbackgrcomp1-tloglikbackgrcomp2 + tloglikspacercomp1 + tloglikspacercomp2 + tloglikpbox2rcomp1 + tloglikpbox2rcomp2;
	_liktreesplit_subtree[is]+=tsum;
	if (bestsplit<0) {
	  bestsplit = is;
	} else if (_liktreesplit_subtree[is]>_liktreesplit_subtree[bestsplit]) {
	  bestsplit = is;
	}
	newcount1 = 0;
	newcount2 = _count_box2nobox2_subtree[_same_count_as_subtree[nsubtrees-1]][0]+_count_box2nobox2_subtree[_same_count_as_subtree[nsubtrees-1]][1];
	if (is!=nsubtrees-1) {
	  if (_same_count_as_subtree[is]>=0) {
	    newcount1 += _count_box2nobox2_subtree[_same_count_as_subtree[is]][0]+_count_box2nobox2_subtree[_same_count_as_subtree[is]][1];
	    newcount2 -= newcount1;
	  }
	}
	cout << "subtree " << is << " tree " << tlogliktree << " seq " << tloglikbox1rcomp1+tloglikbox2rcomp1-tloglikbackgrcomp1 + tloglikbox1rcomp2+tloglikbox2rcomp2-tloglikbackgrcomp2 
	  //<< " box1 " << tloglikbox1rcomp1 << " " << tloglikbox1rcomp2  << " box2 " << tloglikbox2rcomp1 << " " << tloglikbox2rcomp2 << " backg " << tloglikbackgrcomp1 << " " << tloglikbackgrcomp2 
	     << " spacer " << tloglikspacercomp1 + tloglikspacercomp2 << " pbox2 " << tloglikpbox2rcomp1+ tloglikpbox2rcomp2 << " countalloc " << newcount1 << " " << newcount2 << " sum " << _liktreesplit_subtree[is] << endl;
	pselectallowedsplit[nallowedsplit]=_liktreesplit_subtree[is];
	iallowedsplit[nallowedsplit]=is;
	nallowedsplit++;
      }
    }
//     cout << "bestsplit " << bestsplit << " loglik " << _liktreesplit_subtree[bestsplit] << endl;
//     if (bestsplit != original_splitnode) {
//       cout << "bestsplit and original_splitnode differ!" << endl;
//     }
    double psum=0;
    for (int i=0; i<nallowedsplit; i++) {
      pselectallowedsplit[i]=exp(pselectallowedsplit[i]-_liktreesplit_subtree[bestsplit]);
      psum+=pselectallowedsplit[i];
    }
    int rselect = iallowedsplit[sampleProbVect(nallowedsplit, pselectallowedsplit, psum)];
    newcount1 = 0;
    newcount2 = _count_box2nobox2_subtree[_same_count_as_subtree[nsubtrees-1]][0]+_count_box2nobox2_subtree[_same_count_as_subtree[nsubtrees-1]][1];
    if (rselect!=nsubtrees-1) {
      if (_same_count_as_subtree[rselect]>=0) {
	newcount1 += _count_box2nobox2_subtree[_same_count_as_subtree[rselect]][0]+_count_box2nobox2_subtree[_same_count_as_subtree[rselect]][1];
	newcount2 -= newcount1;
      }
    }
    cout << "rselectedsplit " << rselect  << " originalsplit " << original_splitnode << " bestsplit " << bestsplit  << " loglik " << _liktreesplit_subtree[rselect] << " ref_posbox1 " << _ref_posbox1[rcomp1] << " " << _ref_posbox1[rcomp2] << " lbox1 " << _lbox1[rcomp1] << " " << _lbox1[rcomp2] << " lspacer " << _lspacer[rcomp1] << " " << _lspacer[rcomp2] << " lbox2 " << _lbox2[rcomp1] << " " << _lbox2[rcomp2] << " weight " << weighted_compat_pairs[rcomp1][rcomp2] << " origin " << count_alloc_imotif[rcomp1] << " " << count_alloc_imotif[rcomp2] << " new " << newcount1 << " " << newcount2;
    if (rselect != original_splitnode) {
      cout << " differ" << endl;
    } else {
      cout << " same" << endl;
    }   
    delete counts;
    delete pselectallowedsplit;
    delete iallowedsplit;
  }

  for (int ic=0; ic<_dimlikcomput; ic++) {
    delete [] compat_pairs[ic];
    delete [] weighted_compat_pairs[ic];
  }
  delete [] compat_pairs;
  delete [] weighted_compat_pairs;
  delete [] count_alloc_imotif;
}
  

double Mmodel::ComputeLogLikForPbox2(int kbox, int knobox)
{
  double sll = - log(1.0-_minpbox2) + gsl_sf_lngamma(1.0+(double)kbox) 
    + gsl_sf_lngamma(1.0+(double)knobox) 
    - gsl_sf_lngamma(2.0+(double)(kbox+knobox)) 
    +  log(1.0 - gsl_cdf_beta_P(_minpbox2, 1.0+(double)kbox, 1.0+(double)knobox)); 
  //cout << "ComputeLogLikForPbox2 " << kbox << " " << knobox << " " << sll << endl;
  return(sll);
}

double Mmodel::ComputeLogLikForSpacerCount(int* vect, int* vectroot, int dim, bool zeroflag)
{
  double sll = 0;
  int ksum = 0;
  //cout << "spacer";
  for (int ivs=0; ivs<dim; ivs++) {
    int k=0;
    if (vectroot != NULL) {
      k = vectroot[ivs];
      if (!zeroflag) {
	k -= vect[ivs];
      }
    } else if (!zeroflag) {
      k = vect[ivs];
    }
    ksum += k;
    sll += gsl_sf_lngamma(1.0+(double)k) - gsl_sf_lngamma(1.0);
    //cout << " " << k;
  }
  //cout << " " << ksum << endl;
  sll -= gsl_sf_lngamma((double)(dim+ksum)) - gsl_sf_lngamma((int)dim);
  return sll;
}

double Mmodel::ComputeLogLikForBoxCountVectComp1(int** vect, int dim, bool zeroflag)
{
  double sll = 0;
  if (!zeroflag) {
    //cout << "box1";
    for (int ipos=0; ipos<dim; ipos++) {
      int tsum = 0;
      double tll = 0;
      for (int ib=0; ib<4; ib++) {
	int k =vect[ipos][ib];
	tsum += k;
	tll += gsl_sf_lngamma(_alpha_dirich_nuc+(double)k) - gsl_sf_lngamma(_alpha_dirich_nuc);
      }
      tll -= gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc);
      //cout << " " << tsum;
      sll += tll;
    }
    //cout << endl;
  } else {
    sll = 4*(gsl_sf_lngamma(_alpha_dirich_nuc) - gsl_sf_lngamma(_alpha_dirich_nuc)) - (gsl_sf_lngamma(4.0*_alpha_dirich_nuc) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc));
    sll *= dim;
  }
  return sll;
}

double Mmodel::ComputeLogLikForBoxCountVectComp2(int** vect, int** vectroot, int dim, bool zeroflag)
{
  double sll = 0;
  for (int ipos=0; ipos<dim; ipos++) {
    int tsum = 0;
    double tll = 0;
    for (int ib=0; ib<4; ib++) {
      int k = vectroot[ipos][ib];
      if (!zeroflag) {
	k -= vect[ipos][ib];
      }
      tsum += k;
      tll += gsl_sf_lngamma(_alpha_dirich_nuc+(double)k) - gsl_sf_lngamma(_alpha_dirich_nuc);
    }
    tll -= gsl_sf_lngamma(4.0*_alpha_dirich_nuc+(double)tsum) - gsl_sf_lngamma(4.0*_alpha_dirich_nuc);
    //cout << " " << tsum;
    sll += tll;
  }
  return sll;
}

void Mmodel::InitCountsSubtree(int iseq, int rcomp, int compstatus, int* tseq)
{
  for (int ipos=0; ipos<_lmaxbox1; ipos++) {
    for (int ib=0; ib<4; ib++) {
      _count_nuc_matbox1_subtree[iseq][compstatus][ipos][ib] = 0;
    }
  }
  for (int ipos=0; ipos<_lmaxbox2; ipos++) {
    for (int ib=0; ib<4; ib++) {
      _count_nuc_matbox2_subtree[iseq][compstatus][ipos][ib] = 0;
    }
  }
  int b1tmin = tbox1_from_tbox1refpos(_ref_posbox1[rcomp],_alloc_tbox1refpos[iseq]);
  int b1tmax = b1tmin + _lbox1[rcomp] - 1;
  int b1ipos = 0;
  for (int t = b1tmin; t <= b1tmax; t++) {
    if (t>=0 && t<_lseq) {
      _count_nuc_matbox1_subtree[iseq][compstatus][b1ipos][tseq[t]]++; //NODATA
    }
    b1ipos++;
  }
  _loglikbackground_subtree[iseq][compstatus] = 0; 
  if (b1tmax<_lseq) {
    _loglikbackground_subtree[iseq][compstatus] += _logl_cumsum_background_seq[iseq][b1tmax];
  } else {
    _loglikbackground_subtree[iseq][compstatus] += _logl_cumsum_background_seq[iseq][_lseq-1];
  }
  if (b1tmin>0) {
    _loglikbackground_subtree[iseq][compstatus] -= _logl_cumsum_background_seq[iseq][b1tmin-1];
  }

  if (_alloc_varspacer[iseq]>=0) { // otherwise no box2
    int b2tmax = b1tmin - 1 - _lspacer[rcomp] - _alloc_varspacer[iseq];
    int b2tmin = b2tmax - _lbox2[rcomp] + 1;
    int b2ipos = 0;
    for (int t = b2tmin; t<=b2tmax; t++) {
      if (t>=0 && t<_lseq) {
	_count_nuc_matbox2_subtree[iseq][compstatus][b2ipos][tseq[t]]++; //NODATA
      }
      b2ipos++;
    }
    if (b2tmax<_lseq) {
      _loglikbackground_subtree[iseq][compstatus] += _logl_cumsum_background_seq[iseq][b2tmax];
    } else {
      _loglikbackground_subtree[iseq][compstatus] += _logl_cumsum_background_seq[iseq][_lseq-1];
    }
    if (b2tmin>0) {
      _loglikbackground_subtree[iseq][compstatus] -= _logl_cumsum_background_seq[iseq][b2tmin-1];
    }
  }
}

