// TREEMM: a software for identification of bipartite motifs in promoter sequences
// Copyright (C) 2011  Pierre NICOLAS
//
// This file is part of TREEMM.
//
//     TREEMM is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     Foobar is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with TREEMM.  If not, see <http://www.gnu.org/licenses/>.


#include <iostream>
#include <fstream>
#include <cstring>
#include <cstdlib>
#include <gsl/gsl_rng.h>
using namespace std;


#include "Sequences.h"
#include "Mmodel.h"
#include "Tree.h"


int main(int argc, char *argv[])
{
  char* seqfile = NULL;
  char* treefile = NULL;

  int num_arg = 1;
  int nmotifs = -1;
  int niter = -1;
  int nheur = 0;
  int thin = 50;
  double alpha = 1.0;
  unsigned int rseed = time(NULL);
  bool update_p_pos_motif = true;
  int rbackg=3;
  int lmaxbox1 = 30;
  int lmaxbox2 = 30;
  int lminbox1 = 5;
  int lminbox2 = 1;
  int nodupdate = 0;
  bool box2oblig = false;
  while (num_arg < argc)
    {
      if ( (strcmp (argv[num_arg], "-seq") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	seqfile = argv[num_arg];
	cout << "seqfile: " << seqfile << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-tree") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	treefile = argv[num_arg];
	cout << "treefile: " << treefile << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-nmotifs") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	nmotifs = atoi(argv[num_arg]);
	cout << "nmotifs: " << nmotifs << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-niter") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	niter = atoi(argv[num_arg]);
	cout << "niter: " << niter << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-nheur") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	nheur = atoi(argv[num_arg]);
	cout << "nheur: " << nheur << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-thin") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	thin = atoi(argv[num_arg]);
	cout << "thin: " << thin << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-alpha") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	alpha = atof(argv[num_arg]);
	cout << "alpha: " << alpha << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-rseed") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	rseed = atoi(argv[num_arg]);
	cout << "rseed: " << rseed << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-bmm") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	rbackg = atoi(argv[num_arg]);
	cout << "bmm: " << rbackg << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-lmaxbox1") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	lmaxbox1 = atoi(argv[num_arg]);
	cout << "lmaxbox1: " << lmaxbox1 << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-lmaxbox2") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	lmaxbox2 = atoi(argv[num_arg]);
	cout << "lmaxbox2: " << lmaxbox2 << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-lminbox1") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	lminbox1 = atoi(argv[num_arg]);
	cout << "lminbox1: " << lminbox1 << endl;
	num_arg++;	  
	
      } else if ( (strcmp (argv[num_arg], "-lminbox2") == 0) 
	   && (num_arg + 1 < argc) ) {
	
	num_arg++;
	lminbox2 = atoi(argv[num_arg]);
	cout << "lminbox2: " << lminbox2 << endl;
	num_arg++;	  
	
      } else if (strcmp (argv[num_arg], "-box2oblig") == 0) {
	box2oblig = true;
	num_arg++;
	cout << "box2notopt" << endl;		
      } else if (strcmp (argv[num_arg], "-uniformd") == 0) {
	update_p_pos_motif = false;
	num_arg++;
	cout << "uniformd" << endl;	
      } else if ( (strcmp (argv[num_arg], "-nodupdate") == 0)  && (num_arg + 1 < argc)) {
	num_arg++;
	nodupdate = atoi(argv[num_arg]);
	cout << "nodupdate: " << nodupdate << endl;
	num_arg++;
	
      }  else {
	
	cerr << "Usage : " << argv[0] << " "
	     << "-seq <file> [-tree <file>] -nmotifs <#> -niter <#>" << endl
	     << "         [-nheur <#(default 0)>] [-thin <#(default 50)>]" << endl
	     << "         [-alpha <double(default 1.0)>] [-rseed <#>] [-bmm <#(default 3)>]" << endl
	     << "         [-lmaxbox1 <#(default 30)>] [-lmaxbox2 <#(default 30)>]" << endl
	     << "         [-lminbox1 <#(default 5)>] [-lminbox2 <#(default 1)>]" << endl
	     << "         [-uniformd] [-box2oblig]" << endl
	     << endl ;
 	
	return (1) ;
      }
    }
  
  if (seqfile == NULL || nmotifs == -1 || niter == -1) {
    cerr << "Usage : " << argv[0] << " "
	 << "-seq <file> [-tree <file>] -nmotifs <#> -niter <#>" << endl
	 << "         [-nheur <#(default 0)>] [-thin <#(default 50)>]" << endl
	 << "         [-alpha <double(default 1.0)>] [-rseed <#>] [-bmm <#(default 3)>]" << endl
	 << "         [-lmaxbox1 <#(default 30)>] [-lmaxbox2 <#(default 30)>]" << endl
         << "         [-lminbox1 <#(default 5)>] [-lminbox2 <#(default 1)>]" << endl
	 << "         [-uniformd] [-box2oblig]" << endl
	 << endl ;
     return (1) ;
  }


  int lmaxspacer = 20;
  int lminspacer = 5;
  int lmaxvarspacer = 3;
  int lminvarspacer = 2;
  bool allownomotif = true;
  //bool update_probavect = false;
  bool update_probavect = true;

  gsl_rng * r = gsl_rng_alloc(gsl_rng_taus);
  gsl_rng_set(r, rseed);
  
  cout << "random number generator seed: " << rseed << endl;
  
  Sequences* seqs = new Sequences(seqfile, rbackg);
  Tree* tree = NULL;
  if (treefile!=NULL) {
    tree = new Tree(treefile, seqs->NbSeq());
    if (allownomotif) {
      tree->AllocLikelihoodComputation(nmotifs+1, r, alpha);
    } else {
      tree->AllocLikelihoodComputation(nmotifs, r, alpha);
    }
  }
  //tree->printTree();
  //Mmodel* mmodel = new Mmodel(nmotifs, allownomotif, rbackg, lminspacer, lmaxspacer, lvarspacer, lmaxbox, r, NULL);
  Mmodel* mmodel = new Mmodel(nmotifs, allownomotif, alpha, rbackg, lminspacer, lmaxspacer, lminvarspacer, lmaxvarspacer, lminbox1, lmaxbox1, lminbox2, lmaxbox2, box2oblig, r, tree);
  mmodel->MemAllocAlgo(seqs);
  mmodel->Alloc_counts_subtree();
  mmodel->InitAlgo();
  //mmodel->Compute_logl_cumsum_background_seq(seqs);
  //mmodel->Update_pmatbox(seqs); 
  //mmodel->Update_pvarspacer();
  //mmodel->Update_pmotifs();

  mmodel->ComputeLogl_BackgroundMrOnly(seqs);
  ofstream ospar("oparams.txt");
  ofstream osalloc("oalloc.txt");
  for (int iter=0; iter<niter; iter++) {
  //for (int iter=0; iter<100; iter++) {
    if (iter%thin==0) {
      cout << "**** iter " << iter << " ****" << endl;
    }
    //ospar << "**** iter " << iter << " ****" << endl;
    //osalloc << "**** iter " << iter << " ****" << endl;
    mmodel->Compute_logl_cumsum_background_seq(seqs);
    mmodel->Compute_logl_ratio_boxes(seqs);
    mmodel->Compute_ratio_seqt();
    mmodel->Compute_logl_seq_by_motif();
    mmodel->Compute_logltot_and_allocimotif();
    mmodel->Alloc_tbox1refpos_and_varspacer_allseq();
    if (iter%thin==0) {
      ospar << "**** iter " << iter << " ****" << endl;
      osalloc << "**** iter " << iter << " ****" << endl;
      mmodel->printParams(ospar);
      mmodel->printAlloc(osalloc,seqs);
      ospar << "logltot " << mmodel->logltot() << endl;
    }
    mmodel->Update_refpos();
    mmodel->Update_Switchvarspacer();
    mmodel->Update_Sizevarspacer();
    if (!box2oblig) {
      mmodel->Update_Box2isoptional();
    }
    mmodel->Compute_logltot_and_allocimotif();
    mmodel->Alloc_tbox1refpos_and_varspacer_allseq();
    mmodel->Update_pmatbox(seqs); 
    mmodel->Update_pvarspacer();
    mmodel->Update_pmotifs(update_probavect);
    if (update_p_pos_motif && iter>=nodupdate) {
      mmodel->Update_p_pos_motif();
    }
    for (int im=0; im<nmotifs; im++) {
      mmodel->Update_Box2Width(seqs, im);
      mmodel->Update_Box1Width(seqs, im);
      mmodel->Shift_Box1Width(seqs, im);
      mmodel->Shift_Box2Width(seqs, im);
    }
    mmodel->Update_BackgroundMr(seqs);
    //mmodel->simulSeqM1(seqs);
    //return(1);
    if (iter<nheur) {
      mmodel->heuristic_split(seqs);
    } 
    //tree->printTree();
    //mmodel->mergeandsplit(seqs);
    //return(1);
  }
  osalloc.close();
  ospar.close();

  delete seqs;
  gsl_rng_free(r);

  return(1);
}
